@@ -8,8 +8,7 @@ import math
 import platform
 import sys
 import subprocess
-from urllib.request import urlopen, url2pathname, pathname2url
-from urllib.parse import unquote
+from urllib.request import url2pathname, pathname2url
 
 from gi.repository import Gdk
 from gi.repository import Gio
@@ -19,10 +18,11 @@ from gi.repository import GLib
 from pychess.System.Log import log
 from pychess.System import conf, uistuff, prefix
 from pychess.System.debug import print_obj_referrers, print_muppy_sumary
-from pychess.Utils.const import HINT, NAME, SPY
-# from pychess.Utils.checkversion import checkversion
+from pychess.Utils.const import HINT, NAME, SPY, NORMALCHESS
+from pychess.Utils.checkversion import checkversion
 from pychess.widgets import enginesDialog
 from pychess.widgets import newGameDialog
+from pychess.widgets.Background import hexcol
 from pychess.widgets.tipOfTheDay import TipOfTheDay
 from pychess.widgets.discovererDialog import DiscovererDialog
 from pychess.widgets.ExternalsDialog import ExternalsDialog
@@ -30,12 +30,14 @@ from pychess.widgets import gamewidget
 from pychess.widgets.analyzegameDialog import AnalyzeGameDialog
 from pychess.widgets import preferencesDialog, gameinfoDialog, playerinfoDialog
 from pychess.widgets.TaskerManager import internet_game_tasker
+from pychess.widgets.RecentChooser import recent_menu, recent_manager
 from pychess.Players.engineNest import discoverer
 from pychess.Savers import chesspastebin
 from pychess.ic import ICLogon
 from pychess.perspectives import perspective_manager
 from pychess.perspectives.welcome import Welcome
 from pychess.perspectives.games import Games, get_open_dialog
+from pychess.perspectives.learn import Learn
 from pychess.perspectives.fics import FICS
 from pychess.perspectives.database import Database
 from pychess import VERSION, VERSION_NAME
@@ -50,8 +52,6 @@ functionkeys = [Gdk.keyval_from_name(k)
                 for k in ("F1", "F2", "F3", "F4", "F5", "F6", "F7", "F8", "F9",
                           "F10", "F11")]
 
-recentManager = Gtk.RecentManager.get_default()
-
 TARGET_TYPE_URI_LIST = 0xbadbeef
 DRAG_ACTION = Gdk.DragAction.COPY
 DRAG_RESTRICT = Gtk.TargetFlags.OTHER_APP
@@ -110,6 +110,10 @@ class GladeHandlers(object):
             if current_focused_widget is not None and isinstance(current_focused_widget, Gtk.Entry):
                 return False
 
+            # Prevent moving in game while lesson not finished
+            if gmwidg.gamemodel.lesson_game:
+                return
+
             # Navigate on boardview with arrow keys
             if event.keyval in leftkeys:
                 if event.get_state() & Gdk.ModifierType.CONTROL_MASK:
@@ -150,7 +154,7 @@ class GladeHandlers(object):
     def on_recent_game_activated(self, uri):
         if isinstance(uri, str):
             path = url2pathname(uri)
-            recentManager.add_item("file:" + pathname2url(path))
+            recent_manager.add_item("file:" + pathname2url(path))
 
     # Drag 'n' Drop
 
@@ -199,10 +203,13 @@ class GladeHandlers(object):
         if gmwidg is not None:
             ply = gmwidg.board.view.shown
             variation = gmwidg.board.view.shown_variation_idx
-            fen = gmwidg.gamemodel.getBoardAtPly(ply, variation).asFen()
+            board = gmwidg.gamemodel.getBoardAtPly(ply, variation)
+            fen = board.asFen()
+            variant = board.variant
         else:
             fen = None
-        newGameDialog.SetupPositionExtension.run(fen)
+            variant = NORMALCHESS
+        newGameDialog.SetupPositionExtension.run(fen, variant)
 
     def on_enter_game_notation_activate(self, widget):
         newGameDialog.EnterNotationExtension.run()
@@ -252,11 +259,11 @@ class GladeHandlers(object):
     def on_quit1_activate(self, widget, *args):
         perspective = perspective_manager.get_perspective("games")
         if isinstance(widget, Gdk.Event):
-            if len(perspective.gamewidgets) == 1 and conf.get("hideTabs", False):
+            if len(perspective.gamewidgets) == 1 and conf.get("hideTabs"):
                 gmwidg = perspective.cur_gmwidg()
                 perspective.closeGame(gmwidg, gmwidg.gamemodel)
                 return True
-            elif len(perspective.gamewidgets) >= 1 and conf.get("closeAll", False):
+            elif len(perspective.gamewidgets) >= 1 and conf.get("closeAll"):
                 perspective.closeAllGames(perspective.gamewidgets)
                 return True
         if perspective.closeAllGames(perspective.gamewidgets) in (
@@ -346,11 +353,25 @@ class GladeHandlers(object):
 
     def on_copy_pgn_activate(self, widget):
         persp = perspective_manager.get_perspective("games")
-        persp.cur_gmwidg().copy_pgn()
+        if perspective_manager.current_perspective == persp:
+            persp.cur_gmwidg().board.view.copy_pgn()
+            return
+
+        persp = perspective_manager.get_perspective("database")
+        if perspective_manager.current_perspective == persp:
+            if persp.preview_panel is not None:
+                persp.preview_panel.boardview.copy_pgn()
 
     def on_copy_fen_activate(self, widget):
         persp = perspective_manager.get_perspective("games")
-        persp.cur_gmwidg().copy_fen()
+        if perspective_manager.current_perspective == persp:
+            persp.cur_gmwidg().board.view.copy_fen()
+            return
+
+        persp = perspective_manager.get_perspective("database")
+        if perspective_manager.current_perspective == persp:
+            if persp.preview_panel is not None:
+                persp.preview_panel.boardview.copy_fen()
 
     def on_manage_engines_activate(self, widget):
         enginesDialog.run(gamewidget.getWidgets())
@@ -376,6 +397,10 @@ class GladeHandlers(object):
         perspective = perspective_manager.get_perspective("database")
         perspective.on_save_as_clicked(widget)
 
+    def on_create_book_activate(self, widget):
+        perspective = perspective_manager.get_perspective("database")
+        perspective.create_book()
+
     def on_import_endgame_nl_activate(self, widget):
         perspective = perspective_manager.get_perspective("database")
         perspective.on_import_endgame_nl()
@@ -425,18 +450,18 @@ class PyChess(Gtk.Application):
         Gtk.Application.do_startup(self)
 
         if self.purge_recent:
-            items = recentManager.get_items()
+            items = recent_manager.get_items()
             for item in items:
                 uri = item.get_uri()
                 if item.get_application_info("pychess"):
-                    recentManager.remove_item(uri)
+                    recent_manager.remove_item(uri)
 
         self.git_rev = ""
 
         self.initGlade(self.log_viewer)
         self.addPerspectives()
         self.handleArgs(self.chess_file)
-        # checkversion()
+        asyncio.async(checkversion())
 
         self.loaded_cids = {}
         self.saved_cids = {}
@@ -470,18 +495,36 @@ class PyChess(Gtk.Application):
         gamewidget.getWidgets()["leave_fullscreen1"].hide()
 
         # Externals download dialog
-        if not conf.get("dont_show_externals_at_startup", False):
+        if not conf.get("dont_show_externals_at_startup"):
             externals_dialog = ExternalsDialog()
             externals_dialog.show()
 
         # Tip of the day dialog
-        if conf.get("show_tip_at_startup", False):
+        if conf.get("show_tip_at_startup"):
             tip_of_the_day = TipOfTheDay()
             tip_of_the_day.show()
 
+        preferencesDialog.run(gamewidget.getWidgets())
+
+        def on_all_engine_discovered(discoverer):
+            engine = discoverer.getEngineByName(discoverer.getEngineLearn())
+            if engine is None:
+                engine = discoverer.getEngineN(-1)
+            default_engine = engine.get("md5")
+            conf.set("ana_combobox", default_engine)
+            conf.set("inv_ana_combobox", default_engine)
+
+        discoverer.connect_after("all_engines_discovered", on_all_engine_discovered)
+
         dd = DiscovererDialog(discoverer)
         self.dd_task = asyncio.async(dd.start())
 
+        style_ctxt = gamewidget.getWidgets()["main_window"].get_style_context()
+        LIGHT = hexcol(style_ctxt.lookup_color("p_light_color")[1])
+        DARK = hexcol(style_ctxt.lookup_color("p_dark_color")[1])
+        conf.set("lightcolour", LIGHT)
+        conf.set("darkcolour", DARK)
+
     def on_gmwidg_created(self, gamehandler, gmwidg):
         log.debug("GladeHandlers.on_gmwidg_created: starting")
         # Bring playing window to the front
@@ -510,7 +553,7 @@ class PyChess(Gtk.Application):
     def update_recent(self, gamemodel, uri):
         if isinstance(uri, str):
             path = url2pathname(uri)
-            recentManager.add_item("file:" + pathname2url(path))
+            recent_manager.add_item("file:" + pathname2url(path))
 
     def initGlade(self, log_viewer):
         # Init glade and the 'GladeHandlers'
@@ -531,10 +574,10 @@ class PyChess(Gtk.Application):
         for widget in ("hint_mode", "spy_mode"):
             widgets[widget].set_sensitive(False)
 
-        uistuff.keep(widgets["hint_mode"], "hint_mode", first_value=True)
-        uistuff.keep(widgets["spy_mode"], "spy_mode", first_value=True)
-        uistuff.keep(widgets["show_sidepanels"], "show_sidepanels", first_value=True)
-        uistuff.keep(widgets["auto_call_flag"], "autoCallFlag", first_value=True)
+        uistuff.keep(widgets["hint_mode"], "hint_mode")
+        uistuff.keep(widgets["spy_mode"], "spy_mode")
+        uistuff.keep(widgets["show_sidepanels"], "show_sidepanels")
+        uistuff.keep(widgets["auto_call_flag"], "autoCallFlag")
 
         # Show main window and init d'n'd
         widgets["main_window"].set_title('%s - PyChess' % _('Welcome'))
@@ -563,7 +606,7 @@ class PyChess(Gtk.Application):
         # About dialog
         self.aboutdialog = widgets["aboutdialog1"]
         self.aboutdialog.set_program_name(NAME)
-        self.aboutdialog.set_copyright("Copyright © 2006-2017")
+        self.aboutdialog.set_copyright("Copyright © 2006-2018")
         self.aboutdialog.set_version(VERSION_NAME + " " + VERSION)
         if os.path.isdir(prefix.addDataPrefix(".git")):
             try:
@@ -587,33 +630,9 @@ class PyChess(Gtk.Application):
         with open(prefix.addDataPrefix("LICENSE"), encoding="utf-8") as f:
             self.aboutdialog.set_license(f.read())
 
-        # RecentChooser
-        def recent_item_activated(self):
-            uri = self.get_current_uri()
-            try:
-                urlopen(unquote(uri)).close()
-                perspective = perspective_manager.get_perspective("database")
-                perspective.open_chessfile(self.get_current_uri())
-            except (IOError, OSError):
-                # shomething wrong whit the uri
-                recentManager.remove_item(uri)
+        widgets["load_recent_game1"].set_submenu(recent_menu)
 
-        self.menu_recent = Gtk.RecentChooserMenu()
-        self.menu_recent.set_show_tips(True)
-        self.menu_recent.set_sort_type(Gtk.RecentSortType.MRU)
-        self.menu_recent.set_limit(10)
-        self.menu_recent.set_name("menu_recent")
-
-        self.file_filter = Gtk.RecentFilter()
-        self.file_filter.add_mime_type("application/x-chess-pgn")
-        self.file_filter.add_mime_type("application/x-chess-epd")
-        self.file_filter.add_mime_type("application/x-chess-fen")
-        self.menu_recent.set_filter(self.file_filter)
-
-        self.menu_recent.connect("item-activated", recent_item_activated)
-        widgets["load_recent_game1"].set_submenu(self.menu_recent)
-
-        if conf.get("autoLogin", False):
+        if conf.get("autoLogin"):
             internet_game_tasker.connectClicked(None)
 
     def website(self, clb, link):
@@ -621,7 +640,7 @@ class PyChess(Gtk.Application):
 
     def addPerspectives(self):
         perspective_manager.set_widgets(self.widgets)
-        for persp in (Welcome, Games, FICS, Database):
+        for persp in (Welcome, Games, FICS, Database, Learn):
             perspective = persp()
             perspective_manager.add_perspective(perspective)
             perspective.create_toolbuttons()
