@@ -168,11 +168,11 @@ class CECPEngine(ProtocolEngine):
             # we don't start a new game for CECPv2 here,
             # we will do it after feature accept/reject is completed.
 
-    def start(self, event=None):
-        asyncio.async(self.__startBlocking(event))
+    def start(self, event, is_dead):
+        asyncio.async(self.__startBlocking(event, is_dead))
 
     @asyncio.coroutine
-    def __startBlocking(self, event):
+    def __startBlocking(self, event, is_dead):
         if self.protover == 1:
             self.emit("readyForMoves")
             return_value = "ready"
@@ -189,13 +189,13 @@ class CECPEngine(ProtocolEngine):
                 self.emit("readyForMoves")
             except asyncio.TimeoutError:
                 log.warning("Got timeout error", extra={"task": self.defname})
-                raise PlayerIsDead
+                is_dead.add(True)
             except Exception:
                 log.warning("Unknown error", extra={"task": self.defname})
-                raise PlayerIsDead
+                is_dead.add(True)
             else:
                 if return_value == "die":
-                    raise PlayerIsDead
+                    is_dead.add(True)
                 assert return_value == "ready" or return_value == "del"
 
         if event is not None:
@@ -418,7 +418,7 @@ class CECPEngine(ProtocolEngine):
         self.strength = strength
 
         if strength <= 19:
-            self.__setTimeHandicap(0.01 * 10**(strength / 10.))
+            self.__setTimeHandicap(0.01 * 10 ** (strength / 10.))
 
         if strength <= 18:
             self.__setDepth(strength)
@@ -431,8 +431,7 @@ class CECPEngine(ProtocolEngine):
 
         if strength == 20:
             if "gaviota" in self.features["egt"]:
-                self.optionQueue.append("egtpath gaviota %s" % conf.get(
-                    "egtb_path", ""))
+                self.optionQueue.append("egtpath gaviota %s" % conf.get("egtb_path"))
         else:
             self.optionQueue.append("random")
 
@@ -514,6 +513,9 @@ class CECPEngine(ProtocolEngine):
             self.readyForMoveNowCommand = False
 
     def spectatorUndoMoves(self, moves, gamemodel):
+        if self.analyzing_paused:
+            return
+
         log.debug("spectatorUndoMoves: moves=%s gamemodel.ply=%s gamemodel.boards[-1]=%s self.board=%s" % (
             moves, gamemodel.ply, gamemodel.boards[-1], self.board), extra={"task": self.defname})
 
@@ -523,17 +525,22 @@ class CECPEngine(ProtocolEngine):
         self.board = gamemodel.boards[-1]
 
     def playerUndoMoves(self, moves, gamemodel):
-        log.debug("playerUndoMoves: moves=%s gamemodel.ply=%s gamemodel.boards[-1]=%s self.board=%s" % (
-            moves, gamemodel.ply, gamemodel.boards[-1], self.board), extra={"task": self.defname})
+        log.debug("CECPEngine.playerUndoMoves: moves=%s self=%s gamemodel.curplayer=%s" %
+                  (moves, self, gamemodel.curplayer), extra={"task": self.defname})
 
         if gamemodel.curplayer != self and moves % 2 == 1:
             # Interrupt if we were searching, but should no longer do so
+            log.debug("CECPEngine.playerUndoMoves: putting TurnInterrupt into self.move_queue %s" % self.name, extra={"task": self.defname})
             self.queue.put_nowait("int")
 
         self.__tellEngineToStopPlayingCurrentColor()
 
-        for i in range(moves):
-            print("undo", file=self.engine)
+# This caused https://github.com/pychess/pychess/issues/1652
+# so instead of sending "undo" we send "setboard" here
+#        for i in range(moves):
+#            print("undo", file=self.engine)
+
+        self.__setBoard(gamemodel.boards[-1])
 
         if gamemodel.curplayer == self:
             self.board = gamemodel.boards[-1]
@@ -605,16 +612,16 @@ class CECPEngine(ProtocolEngine):
             # king. Therefore we just return the "kill king" move
             # automaticaly
             self.emit("analyze", [(self.board.ply, [toAN(
-                self.board, getMoveKillingKing(self.board))], MATE_VALUE - 1, "", "")])
+                self.board, getMoveKillingKing(self.board))], MATE_VALUE - 1, "1", "")])
             return
 
         print("post", file=self.engine)
         print("analyze", file=self.engine)
         self.engineIsAnalyzing = True
 
-        if not conf.get("infinite_analysis", False):
+        if not conf.get("infinite_analysis"):
             loop = asyncio.get_event_loop()
-            loop.call_later(conf.get("max_analysis_spin", 3), self.__stop_analyze)
+            loop.call_later(conf.get("max_analysis_spin"), self.__stop_analyze)
 
     def __printColor(self):
         if self.features["colors"]:  # or self.mode == INVERSE_ANALYZING:
