@@ -52,6 +52,7 @@ class UCIEngine(ProtocolEngine):
         self.uciPosition = "startpos"
         self.uciPositionListsMoves = False
         self.analysis = [None]
+        self.analysis_depth = None
 
         self.queue = asyncio.Queue()
         self.parse_line_task = asyncio.async(self.parseLine(self.engine))
@@ -68,22 +69,22 @@ class UCIEngine(ProtocolEngine):
     def prestart(self):
         print("uci", file=self.engine)
 
-    def start(self, event=None):
-        asyncio.async(self.__startBlocking(event))
+    def start(self, event, is_dead):
+        asyncio.async(self.__startBlocking(event, is_dead))
 
     @asyncio.coroutine
-    def __startBlocking(self, event):
+    def __startBlocking(self, event, is_dead):
         try:
             return_value = yield from asyncio.wait_for(self.queue.get(), TIME_OUT_SECOND)
         except asyncio.TimeoutError:
             log.warning("Got timeout error", extra={"task": self.defname})
-            raise PlayerIsDead
+            is_dead.add(True)
         except Exception:
             log.warning("Unknown error", extra={"task": self.defname})
-            raise PlayerIsDead
+            is_dead.add(True)
         else:
             if return_value == 'die':
-                raise PlayerIsDead
+                is_dead.add(True)
             assert return_value == "ready" or return_value == "del"
 
         if event is not None:
@@ -310,7 +311,7 @@ class UCIEngine(ProtocolEngine):
 
         if ((not self.hasOption('UCI_Elo')) and
                 (not self.hasOption('Skill Level'))) or strength <= 19:
-            self.timeHandicap = t_hcap = 0.01 * 10**(strength / 10.)
+            self.timeHandicap = t_hcap = 0.01 * 10 ** (strength / 10.)
             self.wtime = int(max(self.wtime * t_hcap, 1))
             self.btime = int(max(self.btime * t_hcap, 1))
             self.incr = int(self.incr * t_hcap)
@@ -319,7 +320,7 @@ class UCIEngine(ProtocolEngine):
             self.setOption('Ponder', strength >= 19 and not forcePonderOff)
 
         if self.hasOption('GaviotaTbPath') and strength == 20:
-            self.setOption('GaviotaTbPath', conf.get("egtb_path", ""))
+            self.setOption('GaviotaTbPath', conf.get("egtb_path"))
 
     # Interacting with the player
 
@@ -372,6 +373,9 @@ class UCIEngine(ProtocolEngine):
             self.queue.put_nowait("int")
 
     def spectatorUndoMoves(self, moves, gamemodel):
+        if self.analyzing_paused:
+            return
+
         log.debug("spectatorUndoMoves: moves=%s \
                   gamemodel.ply=%s \
                   gamemodel.boards[-1]=%s \
@@ -446,16 +450,18 @@ class UCIEngine(ProtocolEngine):
                     # king. Therefore we just return the "kill king" move
                     # automaticaly
                     self.emit("analyze", [(self.board.ply, [toAN(
-                        self.board, getMoveKillingKing(self.board))], MATE_VALUE - 1, "", "")])
+                        self.board, getMoveKillingKing(self.board))], MATE_VALUE - 1, "1", "")])
                     return
                 commands.append("position fen %s" % self.board.asFen())
             else:
                 commands.append("position %s" % self.uciPosition)
 
-            if conf.get("infinite_analysis", False):
+            if self.analysis_depth is not None:
+                commands.append("go depth %s" % self.analysis_depth)
+            elif conf.get("infinite_analysis"):
                 commands.append("go infinite")
             else:
-                move_time = int(conf.get("max_analysis_spin", 3)) * 1000
+                move_time = int(conf.get("max_analysis_spin")) * 1000
                 commands.append("go movetime %s" % move_time)
 
         if self.hasOption("MultiPV") and self.multipvSetting > 1:
@@ -625,7 +631,7 @@ class UCIEngine(ProtocolEngine):
                             #                    print >> self.engine, "stop"
                             if score != 0:
                                 sign = score / abs(score)
-                                score = sign * MATE_VALUE
+                                score = sign * (MATE_VALUE - abs(score))
 
                     movstrs = parts[parts.index("pv") + 1:]
 
@@ -650,7 +656,10 @@ class UCIEngine(ProtocolEngine):
                               extra={"task": self.defname})
                     self.needBestmove = False
                     self.bestmove_event.set()
-                    self.__sendQueuedGo(sendlast=True)
+                    if parts[1] == "(none)":
+                        self.emit("analyze", [])
+                    else:
+                        self.__sendQueuedGo(sendlast=True)
                     continue
 
                 # Stockfish complaining it received a 'stop' without a corresponding 'position..go'
