@@ -8,6 +8,7 @@ from os.path import getmtime
 import platform
 import re
 import sys
+import textwrap
 
 from gi.repository import GLib
 
@@ -24,10 +25,11 @@ from pychess.external.chess_db import Parser
 from pychess.Utils.const import WHITE, BLACK, reprResult, FEN_START, FEN_EMPTY, \
     WON_RESIGN, DRAW, BLACKWON, WHITEWON, NORMALCHESS, DRAW_AGREE, FIRST_PAGE, PREV_PAGE, NEXT_PAGE, \
     ABORTED_REASONS, ADJOURNED_REASONS, WON_CALLFLAG, DRAW_ADJUDICATION, WON_ADJUDICATION, \
-    WHITE_ENGINE_DIED, BLACK_ENGINE_DIED
+    WHITE_ENGINE_DIED, BLACK_ENGINE_DIED, RUNNING
 
 from pychess.System import conf
 from pychess.System.Log import log
+from pychess.System.protoopen import PGN_ENCODING
 from pychess.System.prefix import getEngineDataPrefix
 from pychess.Utils.lutils.LBoard import LBoard
 from pychess.Utils.GameModel import GameModel
@@ -35,14 +37,13 @@ from pychess.Utils.lutils.lmove import toSAN, parseSAN, ParsingError
 from pychess.Utils.Move import Move
 from pychess.Utils.elo import get_elo_rating_change_pgn
 from pychess.Utils.logic import getStatus
-from pychess.Utils.lutils.ldata import MATE_VALUE
 from pychess.Variants import name2variant, NormalBoard, variants
 from pychess.widgets.ChessClock import formatTime
 from pychess.Savers.ChessFile import ChessFile, LoadingError
-from pychess.Savers.database import col2label, TagDatabase
+from pychess.Savers.database import col2label, TagDatabase, parseDateTag
 from pychess.Database import model as dbmodel
-from pychess.Database.PgnImport import TAG_REGEX
-from pychess.Database.model import game, create_indexes, drop_indexes
+from pychess.Database.PgnImport import TAG_REGEX, pgn2Const, PgnImport
+from pychess.Database.model import game, create_indexes, drop_indexes, metadata, ini_schema_version
 
 __label__ = _("Chess Game")
 __ending__ = "pgn"
@@ -72,22 +73,15 @@ pattern = re.compile(r"""
     )    # move (full, count, move with ?!, ?!)
     """, re.VERBOSE | re.DOTALL)
 
-moveeval = re.compile(
-    "\[%eval ([+\-])?(?:#)?(\d+)(?:[,\.](\d{1,2}))?(?:/(\d{1,2}))?\]")
-movetime = re.compile("\[%emt (\d:)?(\d{1,2}:)?(\d{1,4})(?:\.(\d{1,3}))?\]")
+move_eval_re = re.compile("\[%eval\s+([+\-])?(?:#)?(\d+)(?:[,\.](\d{1,2}))?(?:/(\d{1,2}))?\]")
+move_time_re = re.compile("\[%emt\s+(\d:)?(\d{1,2}:)?(\d{1,4})(?:\.(\d{1,3}))?\]")
 
+# Chessbase style circles/arrows {[%csl Ra3][%cal Gc2c3,Rc3d4]}
+comment_circles_re = re.compile("\[%csl\s+((?:[RGBY]\w{2},?)+)\]")
+comment_arrows_re = re.compile("\[%cal\s+((?:[RGBY]\w{4},?)+)\]")
 
-def wrap(string, length):
-    lines = []
-    last = 0
-    while True:
-        if len(string) - last <= length:
-            lines.append(string[last:])
-            break
-        i = string[last:length + last].rfind(" ")
-        lines.append(string[last:i + last])
-        last += i + 1
-    return "\n".join(lines)
+# Mandatory tags (except "Result")
+mandatory_tags = ("Event", "Site", "Date", "Round", "White", "Black")
 
 
 def msToClockTimeTag(ms):
@@ -108,7 +102,7 @@ def parseClockTimeTag(tag):
     Parses 'WhiteClock'/'BlackClock' PGN headers and returns the time the
     player playing that color has left on their clock in milliseconds
     """
-    match = re.match("(\d{1,2}):(\d\d):(\d\d).(\d\d\d)", tag)
+    match = re.match("(\d{1,2}):(\d\d):(\d\d).(\d{1,3})", tag)
     if match:
         hour, minute, sec, msec = match.groups()
         return int(msec) + int(sec) * 1000 + int(minute) * 60 * 1000 + int(
@@ -118,14 +112,14 @@ def parseClockTimeTag(tag):
 def parseTimeControlTag(tag):
     """
     Parses 'TimeControl' PGN header and returns the time and gain the
-    players have on game satrt in seconds
+    players have on game start in seconds
     """
-    match = re.match("(\d+)(?:\+(\d+))?", tag)
+    match = re.match("^(\d+)\+?(\-?\d+)?$", tag)
     if match:
         secs, gain = match.groups()
         return int(secs), int(gain) if gain is not None else 0, 0
     else:
-        match = re.match("(\d+)(?:\/(\d+))?", tag)
+        match = re.match("^(\d+)\/(\d+)$", tag)
         if match:
             moves, secs = match.groups()
             return int(secs), 0, int(moves)
@@ -135,125 +129,137 @@ def parseTimeControlTag(tag):
 
 def save(handle, model, position=None):
     """ Saves the game from GameModel to .pgn """
+    processed_tags = []
 
-    # Header
-    status = "%s" % reprResult[model.status]
+    def write_tag(tag, value, roster=False):
+        nonlocal processed_tags
+        if tag in processed_tags or (not roster and not value):
+            return
+        try:
+            pval = str(value)
+            pval = pval.replace("\\", "\\\\")
+            pval = pval.replace("\"", "\\\"")
+            print('[%s "%s"]' % (tag, pval), file=handle)
+        except UnicodeEncodeError:
+            pval = bytes(pval, "utf-8").decode(PGN_ENCODING, errors="ignore")
+            print('[%s "%s"]' % (tag, pval), file=handle)
+        processed_tags = processed_tags + [tag]
 
-    print('[Event "%s"]' % model.getTagExport("Event", ""), file=handle)
-    print('[Site "%s"]' % model.getTagExport("Site", ""), file=handle)
-    print('[Date "%04d.%02d.%02d"]' %
-          (int(model.tags["Year"]), int(model.tags["Month"]), int(model.tags["Day"])), file=handle)
-    print('[Round "%s"]' % model.getTagExport("Round", "?"), file=handle)
-    print('[White "%s"]' % repr(model.players[WHITE]), file=handle)
-    print('[Black "%s"]' % repr(model.players[BLACK]), file=handle)
-    print('[Result "%s"]' % status, file=handle)
-    tag = model.getTagExport("ECO", "")
-    if tag != "":
-        print('[ECO "%s"]' % tag, file=handle)
-        tag = model.getTagExport("Opening", "")
-        if tag != "":
-            print('[Opening "%s"]' % tag, file=handle)  # Unofficial
-            tag = model.getTagExport("Variation", "")
-            if tag != "":
-                print('[Variation "%s"]' % tag, file=handle)  # Unofficial
-    welo = model.getTagExport("WhiteElo", "")
-    if welo != "":
-        print('[WhiteElo "%s"]' % welo, file=handle)
-    belo = model.getTagExport("BlackElo", "")
-    if belo != "":
-        print('[BlackElo "%s"]' % belo, file=handle)
-    if welo != "" and belo != "" and conf.get("saveRatingChange", False):
-        diff = str(get_elo_rating_change_pgn(model, WHITE))
-        if diff != "":
-            print('[WhiteRatingDiff "%s"]' % diff, file=handle)  # Unofficial
-        diff = str(get_elo_rating_change_pgn(model, BLACK))
-        if diff != "":
-            print('[BlackRatingDiff "%s"]' % diff, file=handle)  # Unofficial
-    tag = model.getTagExport("TimeControl", "")
-    if tag != "":
-        print('[TimeControl "%s"]' % tag, file=handle)
-    if model.timed:
-        print('[WhiteClock "%s"]' %
-              msToClockTimeTag(int(model.timemodel.getPlayerTime(WHITE) * 1000)), file=handle)
-        print('[BlackClock "%s"]' %
-              msToClockTimeTag(int(model.timemodel.getPlayerTime(BLACK) * 1000)), file=handle)
+    # Mandatory ordered seven-tag roster
+    status = reprResult[model.status]
+    for tag in mandatory_tags:
+        value = model.tags[tag]
+        if tag == "Date":
+            y, m, d = parseDateTag(value)
+            y = "%04d" % y if y is not None else "????"
+            m = "%02d" % m if m is not None else "??"
+            d = "%02d" % d if d is not None else "??"
+            value = "%s.%s.%s" % (y, m, d)
+        elif value == "":
+            value = "?"
+        write_tag(tag, value, roster=True)
+    write_tag("Result", reprResult[model.status], roster=True)
 
+    # Variant
     if model.variant.variant != NORMALCHESS:
-        print('[Variant "%s"]' % model.variant.cecp_name.capitalize(),
-              file=handle)
+        write_tag("Variant", model.variant.cecp_name.capitalize())
 
+    # Initial position
     if model.boards[0].asFen() != FEN_START:
-        print('[SetUp "1"]', file=handle)
-        print('[FEN "%s"]' % model.boards[0].asFen(), file=handle)
-    print('[PlyCount "%s"]' % (model.ply - model.lowply), file=handle)
-    tag = model.getTagExport("Annotator", "")
-    if tag != "":
-        print('[Annotator "%s"]' % tag, file=handle)
-    if model.reason == WON_CALLFLAG:
-        termination = "time forfeit"
-    elif model.reason == WON_ADJUDICATION and model.isEngine2EngineGame():
-        termination = "rules infraction"
-    elif model.reason in (DRAW_ADJUDICATION, WON_ADJUDICATION):
-        termination = "adjudication"
-    elif model.reason == WHITE_ENGINE_DIED:
-        termination = "white engine died"
-    elif model.reason == BLACK_ENGINE_DIED:
-        termination = "black engine died"
-    elif model.reason in ABORTED_REASONS:
-        termination = "abandoned"
-    elif model.reason in ADJOURNED_REASONS:
-        termination = "unterminated"
-    else:
-        termination = "normal"
-    print('[Termination "%s"]' % termination, file=handle)
+        write_tag("SetUp", "1")
+        write_tag("FEN", model.boards[0].asFen())
 
-    save_emt = conf.get("saveEmt", False)
-    save_eval = conf.get("saveEval", False)
+    # Number of moves
+    write_tag("PlyCount", model.ply - model.lowply)
+
+    # Final position
+    if model.reason == WON_CALLFLAG:
+        value = "time forfeit"
+    elif model.reason == WON_ADJUDICATION and model.isEngine2EngineGame():
+        value = "rules infraction"
+    elif model.reason in (DRAW_ADJUDICATION, WON_ADJUDICATION):
+        value = "adjudication"
+    elif model.reason == WHITE_ENGINE_DIED:
+        value = "white engine died"
+    elif model.reason == BLACK_ENGINE_DIED:
+        value = "black engine died"
+    elif model.reason in ABORTED_REASONS:
+        value = "abandoned"
+    elif model.reason in ADJOURNED_REASONS:
+        value = "unterminated"
+    else:
+        value = "unterminated" if status == "*" else None
+    if value is not None:
+        write_tag("Termination", value)
+
+    # ELO and its variation
+    if conf.get("saveRatingChange"):
+        welo = model.tags["WhiteElo"]
+        belo = model.tags["BlackElo"]
+        if welo != "" and belo != "":
+            write_tag("WhiteRatingDiff", get_elo_rating_change_pgn(model, WHITE))  # Unofficial
+            write_tag("BlackRatingDiff", get_elo_rating_change_pgn(model, BLACK))  # Unofficial
+
+    # Time
+    if model.timed:
+        write_tag('WhiteClock', msToClockTimeTag(int(model.timemodel.getPlayerTime(WHITE) * 1000)))
+        write_tag('BlackClock', msToClockTimeTag(int(model.timemodel.getPlayerTime(BLACK) * 1000)))
+
+    # Write all the unprocessed tags
+    for tag in model.tags:
+        # Debug: print(">> %s = %s" % (tag, str(model.tags[tag])))
+        write_tag(tag, model.tags[tag])
 
     # Discovery of the moves and comments
+    save_emt = conf.get("saveEmt")
+    save_eval = conf.get("saveEval")
     result = []
     walk(model.boards[0].board, result, model, save_emt, save_eval)
 
     # Alignment of the fetched elements
-    indented = conf.get("indentPgn", False)
-    buffer = ""
-    depth = 0
-    crlf = False
-    for text in result:
-        # De/Indentation
-        crlf = (buffer[-1:] if len(buffer) > 0 else "") in ["\r", "\n"]
-        if text == "(":
-            depth += 1
-            if indented and not crlf:
+    indented = conf.get("indentPgn")
+    if indented:
+        buffer = ""
+        depth = 0
+        crlf = False
+        for text in result:
+            # De/Indentation
+            crlf = (buffer[-1:] if len(buffer) > 0 else "") in ["\r", "\n"]
+            if text == "(":
+                depth += 1
+                if indented and not crlf:
+                    buffer += os.linesep
+                    crlf = True
+            # Space between each term
+            last = buffer[-1:] if len(buffer) > 0 else ""
+            crlf = last in ["\r", "\n"]
+            if not crlf and last != " " and last != "\t" and last != "(" and not text.startswith("\r") and not text.startswith("\n") and text != ")" and len(buffer) > 0:
+                buffer += " "
+            # New line for a new main move
+            if len(buffer) == 0 or (indented and depth == 0 and last != "\r" and last != "\n" and re.match("^[0-9]+\.", text) is not None):
                 buffer += os.linesep
                 crlf = True
-        # Space between each term
-        last = buffer[-1:] if len(buffer) > 0 else ""
-        crlf = last in ["\r", "\n"]
-        if not crlf and last != " " and last != "\t" and last != "(" and not text.startswith("\r") and not text.startswith("\n") and text != ")" and len(buffer) > 0:
-            buffer += " "
-        # New line for a new main move
-        if len(buffer) == 0 or (indented and depth == 0 and last != "\r" and last != "\n" and re.match("^[0-9]+\.", text) is not None):
-            buffer += os.linesep
-            crlf = True
-        # Alignment
-        if crlf and depth > 0:
-            for j in range(0, depth):
-                buffer += "    "
-        # Term
-        buffer += text
-        if indented and text == ")":
-            buffer += os.linesep
-            crlf = True
-            depth -= 1
-
-    # Status of the game
-    buffer += ("" if crlf else (os.linesep if indented else " ")) + status
-    if not indented:
-        buffer = wrap(buffer, 80)
+            # Alignment
+            if crlf and depth > 0:
+                for j in range(0, depth):
+                    buffer += "    "
+            # Term
+            buffer += text
+            if indented and text == ")":
+                buffer += os.linesep
+                crlf = True
+                depth -= 1
+    else:
+        # Add new line to separate tag section and movetext
+        print('', file=handle)
+        buffer = textwrap.fill(" ".join(result), width=80)
 
     # Final
-    print(buffer, "", file=handle)
+    status = reprResult[model.status]
+    print(buffer, status, file=handle)
+    # Add new line to separate next game
+    print('', file=handle)
+
     output = handle.getvalue() if isinstance(handle, StringIO) else ""
     handle.close()
     return output
@@ -308,7 +314,6 @@ def walk(node, result, model, save_emt=False, save_eval=False, vari=False):
 
         for child in node.children:
             if isinstance(child, str):
-                child = re.sub("\[%.*?\]", "", child)
                 # comment
                 if child:
                     result.append("{%s}" % child)
@@ -443,20 +448,31 @@ class PGNFile(ChessFile):
         self.tag_database.close()
         ChessFile.close(self)
 
-    def init_tag_database(self, importer):
+    def init_tag_database(self, importer=None):
         """ Create/open .sqlite database of game header tags """
         # Import .pgn header tags to .sqlite database
+
+        sqlite_path = self.path.replace(".pgn", ".sqlite")
+        if os.path.isfile(self.path) and os.path.isfile(sqlite_path) and getmtime(self.path) > getmtime(sqlite_path):
+            metadata.drop_all(self.engine)
+            metadata.create_all(self.engine)
+            ini_schema_version(self.engine)
+
         size = self.size
         if size > 0 and self.tag_database.count == 0:
             if size > 10000000:
                 drop_indexes(self.engine)
             if self.progressbar is not None:
                 GLib.idle_add(self.progressbar.set_text, _("Importing game headers..."))
+            if importer is None:
+                importer = PgnImport(self)
             importer.initialize()
             importer.do_import(self.path, progressbar=self.progressbar)
             if size > 10000000 and not importer.cancel:
                 create_indexes(self.engine)
 
+        return importer
+
     def init_chess_db(self):
         """ Create/open polyglot .bin file with extra win/loss/draw stats
             using chess_db parser from https://github.com/mcostalba/chess_db
@@ -521,10 +537,10 @@ class PGNFile(ChessFile):
         self.tag_database.build_order_by(self.order_col, self.is_desc)
 
     def reset_last_seen(self):
-        col_max = "ZZZ" if isinstance(self.order_col.type, String) else 2**32
+        col_max = "ZZZ" if isinstance(self.order_col.type, String) else 2 ** 32
         col_min = "" if isinstance(self.order_col.type, String) else -1
         if self.is_desc:
-            self.last_seen = [(col_max, 2**32)]
+            self.last_seen = [(col_max, 2 ** 32)]
         else:
             self.last_seen = [(col_min, -1)]
 
@@ -666,7 +682,10 @@ class PGNFile(ChessFile):
             if line.startswith('[') and line.endswith(']'):
                 tag_match = TAG_REGEX.match(line)
                 if tag_match:
-                    header[tag_match.group(1)] = tag_match.group(2)
+                    value = tag_match.group(2)
+                    value = value.replace("\\\"", "\"")
+                    value = value.replace("\\\\", "\\")
+                    header[tag_match.group(1)] = value
             else:
                 break
         return header
@@ -674,48 +693,35 @@ class PGNFile(ChessFile):
     def loadToModel(self, rec, position=-1, model=None):
         """ Parse game text and load game record header tags to a GameModel object """
 
-        if not model:
+        if model is None:
             model = GameModel()
 
         if self.pgn_is_string:
             rec = self.games[0]
-            game_date = rec["Date"]
-            result = rec["Result"]
+
+        # Load mandatory tags
+        for tag in mandatory_tags:
+            model.tags[tag] = rec[tag]
+
+        # Load other tags
+        for tag in ('WhiteElo', 'BlackElo', 'ECO', 'TimeControl', 'Annotator'):
+            model.tags[tag] = rec[tag]
+
+        if self.pgn_is_string:
+            for tag in rec:
+                if isinstance(rec[tag], str) and rec[tag]:
+                    model.tags[tag] = rec[tag]
+        else:
+            model.info = self.tag_database.get_info(rec)
+            extra_tags = self.tag_database.get_exta_tags(rec)
+            for et in extra_tags:
+                model.tags[et['tag_name']] = et['tag_value']
+
+        if self.pgn_is_string:
             variant = rec["Variant"].capitalize()
         else:
-            game_date = self.get_date(rec)
-            result = reprResult[rec["Result"]]
             variant = self.get_variant(rec)
 
-        # the seven mandatory PGN headers
-        model.tags['Event'] = rec["Event"] if rec["Event"] is not None else ""
-        model.tags['Site'] = rec["Site"] if rec["Site"] is not None else ""
-        model.tags['Date'] = game_date
-        model.tags['Round'] = rec["Round"]
-        model.tags['White'] = rec["White"] if rec["White"] is not None else "?"
-        model.tags['Black'] = rec["Black"] if rec["Black"] is not None else "?"
-        model.tags['Result'] = result
-
-        if model.tags['Date']:
-            date_match = re.match(".*(\d{4}).(\d{2}).(\d{2}).*",
-                                  model.tags['Date'])
-            if date_match:
-                year, month, day = date_match.groups()
-                model.tags['Year'] = year
-                model.tags['Month'] = month
-                model.tags['Day'] = day
-
-        # non-mandatory tags
-        for tag in ('Annotator', 'ECO', 'WhiteElo', 'BlackElo', 'TimeControl'):
-            value = rec[tag]
-            if value:
-                model.tags[tag] = value
-            else:
-                model.tags[tag] = ""
-
-        if not self.pgn_is_string:
-            model.info = self.tag_database.get_info(rec)
-
         if model.tags['TimeControl']:
             tc = parseTimeControlTag(model.tags['TimeControl'])
             if tc is not None:
@@ -726,9 +732,9 @@ class PGNFile(ChessFile):
                 model.timemodel.minutes = secs / 60
                 model.timemodel.moves = moves
                 for tag, color in (('WhiteClock', WHITE), ('BlackClock', BLACK)):
-                    if hasattr(rec, tag):
+                    if tag in model.tags:
                         try:
-                            millisec = parseClockTimeTag(rec[tag])
+                            millisec = parseClockTimeTag(model.tags[tag])
                             # We need to fix when FICS reports negative clock time like this
                             # [TimeControl "180+0"]
                             # [WhiteClock "0:00:15.867"]
@@ -740,7 +746,6 @@ class PGNFile(ChessFile):
                         except ValueError:
                             raise LoadingError(
                                 "Error parsing '%s'" % tag)
-
         fenstr = rec["FEN"]
 
         if variant:
@@ -767,6 +772,7 @@ class PGNFile(ChessFile):
         if fenstr:
             try:
                 board.applyFen(fenstr)
+                model.tags["FEN"] = fenstr
             except SyntaxError as err:
                 board.applyFen(FEN_EMPTY)
                 raise LoadingError(
@@ -845,7 +851,7 @@ class PGNFile(ChessFile):
                 for child in board.children:
                     if isinstance(child, str):
                         if self.has_emt:
-                            match = movetime.search(child)
+                            match = move_time_re.search(child)
                             if match:
                                 movecount, color = divmod(ply + 1, 2)
                                 hour, minute, sec, msec = match.groups()
@@ -854,19 +860,15 @@ class PGNFile(ChessFile):
                                 hour = 0 if hour is None else int(hour[:-1])
                                 minute = 0 if minute is None else int(minute[:-1])
                                 msec = 0 if msec is None else int(msec)
-                                msec += int(sec) * 1000 + int(
-                                    minute) * 60 * 1000 + int(
-                                        hour) * 60 * 60 * 1000
-                                model.timemodel.intervals[color][
-                                    movecount] = prev - msec / 1000. + gain
+                                msec += int(sec) * 1000 + int(minute) * 60 * 1000 + int(hour) * 60 * 60 * 1000
+                                model.timemodel.intervals[color][movecount] = prev - msec / 1000. + gain
 
                         if self.has_eval:
-                            match = moveeval.search(child)
+                            match = move_eval_re.search(child)
                             if match:
                                 sign, num, fraction, depth = match.groups()
                                 sign = 1 if sign is None or sign == "+" else -1
-                                num = int(num) if int(
-                                    num) == MATE_VALUE else int(num)
+                                num = int(num)
                                 fraction = 0 if fraction is None else int(
                                     fraction)
                                 value = sign * (num * 100 + fraction)
@@ -882,7 +884,15 @@ class PGNFile(ChessFile):
 
         # Apply result from .pgn if the last position was loaded
         if position == -1 or len(model.moves) == position - model.lowply:
-            status = rec["Result"]
+            if self.pgn_is_string:
+                result = rec["Result"]
+                if result in pgn2Const:
+                    status = pgn2Const[result]
+                else:
+                    status = RUNNING
+            else:
+                status = rec["Result"]
+
             if status in (WHITEWON, BLACKWON) and status != model.status:
                 model.status = status
                 model.reason = WON_RESIGN
@@ -890,6 +900,9 @@ class PGNFile(ChessFile):
                 model.status = DRAW
                 model.reason = DRAW_AGREE
 
+        if model.timed:
+            model.timemodel.movingColor = model.boards[-1].color
+
         # If parsing gave an error we throw it now, to enlarge our possibility
         # of being able to continue the game from where it failed.
         if self.error:
@@ -1005,8 +1018,12 @@ class PGNFile(ChessFile):
 
                 elif group == COMMENT_BRACE:
                     comm = text.replace('{\r\n', '{').replace('\r\n}', '}')
-                    comm = comm[1:-1].splitlines()
-                    comment = ' '.join([line.strip() for line in comm])
+                    # Preserve new lines of lichess study comments
+                    if self.path is not None and "lichess_study_" in self.path:
+                        comment = comm[1:-1]
+                    else:
+                        comm = comm[1:-1].splitlines()
+                        comment = ' '.join([line.strip() for line in comm])
                     if variation and last_board == board:
                         # initial variation comment
                         boards[0].children.append(comment)
@@ -1052,34 +1069,25 @@ class PGNFile(ChessFile):
             if (not in_comment and "{" in line) or (in_comment and "}" in line):
                 in_comment = line.rfind("{") > line.rfind("}")
 
+            # if there is something add it
             if line.strip():
+                if not self.pgn_is_string and self.handle.pgn_encoding != PGN_ENCODING:
+                    line = line.encode(PGN_ENCODING).decode(self.handle.pgn_encoding)
                 lines.append(line)
                 line = self.handle.readline()
-            elif len(lines) == 0:
+            # if line is empty it should be the game separator line except...
+            elif len(lines) == 0 or in_comment:
+                if in_comment:
+                    lines.append(line)
                 line = self.handle.readline()
             else:
                 break
-
         return "".join(lines)
 
     def get_variant(self, rec):
         variant = rec["Variant"]
         return variants[variant].cecp_name.capitalize() if variant else ""
 
-    def get_date(self, rec):
-        year = rec['Year']
-        month = rec['Month']
-        day = rec['Day']
-        if year and month and day:
-            tag_date = "%s.%02d.%02d" % (year, month, day)
-        elif year and month:
-            tag_date = "%s.%02d" % (year, month)
-        elif year:
-            tag_date = "%s" % year
-        else:
-            tag_date = ""
-        return tag_date
-
 
 nag2symbolDict = {
     "$0": "",
