@@ -42,6 +42,9 @@ class Board:
             self.board = lboard
         self.board.pieceBoard = self
 
+        # Set True in interactive lesson games after they happened
+        self.played = False
+
         if setup:
             if lboard is None:
                 if setup is True:
@@ -158,7 +161,7 @@ class Board:
                                 move.flag in (QUEEN_CASTLE, KING_CASTLE)):
             piece = PAWN if self.variant == CRAZYHOUSECHESS and self[
                 cord1].promoted else self[cord1].piece
-            cord = self.newHoldingCord(self.color)
+            cord = board1.getHoldingCord(self.color, piece)
             moved.append((board1[cord], cord1))
             # add all captured pieces to "new" list to enforce repainting them after a possible reordering
             new = board1.getHoldingPieces(self.color)
