@@ -1,7 +1,7 @@
-from collections import defaultdict
 import asyncio
-import traceback
+import collections
 import datetime
+import traceback
 from queue import Queue
 from io import StringIO
 
@@ -29,7 +29,7 @@ from .const import WAITING_TO_START, UNKNOWN_REASON, WHITE, ARTIFICIAL, RUNNING,
     ADJOURNED_AGREEMENT, PAUSE_OFFER, RESUME_OFFER, ACTION_ERROR_NONE_TO_ACCEPT, \
     ABORTED_AGREEMENT, WHITE_ENGINE_DIED, BLACK_ENGINE_DIED, WON_ADJUDICATION, \
     UNDOABLE_STATES, DRAW_REPITITION, UNDOABLE_REASONS, UNFINISHED_STATES, \
-    DRAW_50MOVES
+    DRAW_50MOVES, HINT
 
 
 class GameModel(GObject.GObject):
@@ -43,11 +43,17 @@ class GameModel(GObject.GObject):
         # game_changed is emitted when a move has been made.
         "game_changed": (GObject.SignalFlags.RUN_FIRST, None, (int, )),
         # moves_undoig is emitted when a undoMoves call has been accepted, but
-        # before anywork has been done to execute it.
+        # before any work has been done to execute it.
         "moves_undoing": (GObject.SignalFlags.RUN_FIRST, None, (int, )),
         # moves_undone is emitted after n moves have been undone in the
         # gamemodel and the players.
         "moves_undone": (GObject.SignalFlags.RUN_FIRST, None, (int, )),
+        # variation_undoig is emitted when a undo_in_variation call has been started, but
+        # before any work has been done to execute it.
+        "variation_undoing": (GObject.SignalFlags.RUN_FIRST, None, ()),
+        # variation_undone is emitted after 1 move have been undone in the
+        # boardview shown variation
+        "variation_undone": (GObject.SignalFlags.RUN_FIRST, None, ()),
         # game_unended is emitted if moves have been undone, such that the game
         # which had previously ended, is now again active.
         "game_unended": (GObject.SignalFlags.RUN_FIRST, None, ()),
@@ -126,21 +132,16 @@ class GameModel(GObject.GObject):
             self.timemodel = timemodel
         self.timemodel.gamemodel = self
 
-        self.connections = defaultdict(list)  # mainly for IC subclasses
+        self.connections = collections.defaultdict(list)  # mainly for IC subclasses
         self.analyzer_cids = {}
         self.examined = False
 
         now = datetime.datetime.now()
-        self.tags = {
-            "Event": _("Local Event"),
-            "Site": _("Local Site"),
-            "Round": 1,
-            "Year": now.year,
-            "Month": now.month,
-            "Day": now.day,
-            "Time": "%02d:%02d:00" % (now.hour, now.minute),
-            "Result": "*",
-        }
+        self.tags = collections.defaultdict(str)
+        self.tags["Event"] = _("Local Event")
+        self.tags["Site"] = _("Local Site")
+        self.tags["Date"] = "%04d.%02d.%02d" % (now.year, now.month, now.day)
+        self.tags["Round"] = "1"
 
         self.endstatus = None
         self.zero_reached_cid = None
@@ -149,7 +150,7 @@ class GameModel(GObject.GObject):
         if self.timed:
             self.zero_reached_cid = self.timemodel.connect('zero_reached', self.zero_reached)
             if self.timemodel.moves == 0:
-                self.tags["TimeControl"] = "%d+%d" % (self.timemodel.minutes * 60, self.timemodel.gain)
+                self.tags["TimeControl"] = "%d%s%d" % (self.timemodel.minutes * 60, "+" if self.timemodel.gain >= 0 else "-", abs(self.timemodel.gain))
             else:
                 self.tags["TimeControl"] = "%d/%d" % (self.timemodel.moves, self.timemodel.minutes * 60)
             # Notice: tags["WhiteClock"] and tags["BlackClock"] are never set
@@ -173,8 +174,30 @@ class GameModel(GObject.GObject):
 
         self.undoQueue = Queue()
 
+        # learn_type set by LearnModel.set_learn_data()
+        self.offline_lecture = False
+        self.puzzle_game = False
+        self.lesson_game = False
+        self.end_game = False
+        self.solved = False
+
+    @property
+    def practice_game(self):
+        return self.puzzle_game or self.end_game
+
+    @property
+    def starting_color(self):
+        return BLACK if "FEN" in self.tags and self.tags["FEN"].split()[1] == "b" else WHITE
+
+    @property
+    def orientation(self):
+        if "Orientation" in self.tags:
+            return BLACK if self.tags["Orintation"].lower() == "black" else WHITE
+        else:
+            return self.starting_color
+
     def zero_reached(self, timemodel, color):
-        if conf.get('autoCallFlag', True) and self.players[1 - color].__type__ == ARTIFICIAL:
+        if conf.get('autoCallFlag') and self.players[1 - color].__type__ == ARTIFICIAL:
             if self.status == RUNNING and timemodel.getPlayerTime(color) <= 0:
                 log.info(
                     'Automatically sending flag call on behalf of player %s.' %
@@ -226,19 +249,6 @@ class GameModel(GObject.GObject):
         log.debug("GameModel.setPlayers: <- emit players_changed")
         log.debug("GameModel.setPlayers: returning")
 
-    def getTag(self, name, defaultValue):
-        if name in self.tags and self.tags[name] is not None:
-            return self.tags[name]
-        else:
-            return defaultValue
-
-    def getTagExport(self, name, defaultValue):
-        val = self.getTag(name, defaultValue)
-        if type(val) is str:
-            val = val.replace("\\", "\\\\")
-            val = val.replace("\"", "\\\"")
-        return val
-
     def color(self, player):
         if player is self.players[0]:
             return WHITE
@@ -246,18 +256,27 @@ class GameModel(GObject.GObject):
             return BLACK
 
     @asyncio.coroutine
-    def start_analyzer(self, analyzer_type):
+    def start_analyzer(self, analyzer_type, force_engine=None):
+        # Don't start regular analyzers
+        if (self.practice_game or self.lesson_game) and force_engine is None and not self.solved:
+            return
+
         # prevent starting new analyzers again and again
         # when fics lecture reuses the same gamemodel
         if analyzer_type in self.spectators:
             return
 
         from pychess.Players.engineNest import init_engine
-        analyzer = yield from init_engine(analyzer_type, self)
+        analyzer = yield from init_engine(analyzer_type, self, force_engine=force_engine)
         if analyzer is None:
             return
 
         analyzer.setOptionInitialBoard(self)
+        # Enable to find alternate hint in learn perspective puzzles
+        if force_engine is not None:
+            analyzer.setOption("MultiPV", 3)
+            analyzer.analysis_depth = 20
+
         self.spectators[analyzer_type] = analyzer
         self.emit("analyzer_added", analyzer, analyzer_type)
         self.analyzer_cids[analyzer_type] = analyzer.connect("analyze", self.on_analyze)
@@ -297,9 +316,21 @@ class GameModel(GObject.GObject):
         yield from self.start_analyzer(analyzer_type)
 
     def on_analyze(self, analyzer, analysis):
+        if analysis and (self.practice_game or self.lesson_game):
+            for i, anal in enumerate(analysis):
+                if anal is not None:
+                    ply, pv, score, depth, nps = anal
+                    if len(pv) > 0:
+                        if ply not in self.hints:
+                            self.hints[ply] = []
+
+                        if len(self.hints[ply]) < i + 1:
+                            self.hints[ply].append((pv[0], score))
+                        else:
+                            self.hints[ply][i] = (pv[0], score)
         if analysis and analysis[0] is not None:
             ply, pv, score, depth, nps = analysis[0]
-            if score is not None:
+            if score is not None and depth:
                 if analyzer.mode == ANALYZING:
                     if (ply not in self.scores) or (int(self.scores[ply][2]) <= int(depth)):
                         self.scores[ply] = (pv, score, depth)
@@ -363,10 +394,6 @@ class GameModel(GObject.GObject):
         return index
 
     def getBoardAtPly(self, ply, variation=0):
-        # Losing on time in FICS game will undo our last move if it was taken
-        # too late
-        if variation == 0 and ply > self.ply:
-            ply = self.ply
         try:
             return self.variations[variation][self._plyToIndex(ply)]
         except IndexError:
@@ -408,8 +435,7 @@ class GameModel(GObject.GObject):
         return not self.hasLocalPlayer()
 
     def isEngine2EngineGame(self):
-        if self.players[0].__type__ == ARTIFICIAL and self.players[
-                1].__type__ == ARTIFICIAL:
+        if len(self.players) == 2 and self.players[0].__type__ == ARTIFICIAL and self.players[1].__type__ == ARTIFICIAL:
             return True
         else:
             return False
@@ -418,9 +444,9 @@ class GameModel(GObject.GObject):
         if self.players and self.status in (WAITING_TO_START, PAUSED, RUNNING):
             if (self.players[0].__type__ == LOCAL and self.players[1].__type__ == REMOTE) or \
                (self.players[1].__type__ == LOCAL and self.players[0].__type__ == REMOTE) or \
+               ((self.offline_lecture or self.practice_game or self.lesson_game) and not self.solved) or \
                (self.players[1].__type__ == REMOTE and self.players[0].__type__ == REMOTE and
                     self.examined and (
-                    self.connection.offline_lecture or
                     self.players[0].name == "puzzlebot" or self.players[1].name == "puzzlebot") or
                     self.players[0].name == "endgamebot" or self.players[1].name == "endgamebot"):
                 return True
@@ -535,9 +561,8 @@ class GameModel(GObject.GObject):
             if offer.type == DRAW_OFFER:
                 self.end(DRAW, DRAW_AGREE)
             elif offer.type == TAKEBACK_OFFER:
-                log.debug("GameModel.acceptReceived: undoMoves(%s)" % (
-                    self.ply - offer.param))
-                self.undoMoves(self.ply - offer.param)
+                log.debug("GameModel.acceptReceived: undoMoves(%s)" % offer.param)
+                self.undoMoves(offer.param)
             elif offer.type == ADJOURN_OFFER:
                 self.end(ADJOURNED, ADJOURNED_AGREEMENT)
             elif offer.type == ABORT_OFFER:
@@ -621,7 +646,6 @@ class GameModel(GObject.GObject):
     # Run stuff
 
     def start(self):
-
         def coro():
             log.debug("GameModel.run: Starting. self=%s" % self)
             # Avoid racecondition when self.start is called while we are in
@@ -636,14 +660,25 @@ class GameModel(GObject.GObject):
 
             for player in self.players + list(self.spectators.values()):
                 event = asyncio.Event()
-                player.start(event)
+                is_dead = set()
+                player.start(event, is_dead)
+
                 yield from event.wait()
 
+                if is_dead:
+                    if player in self.players[WHITE]:
+                        self.kill(WHITE_ENGINE_DIED)
+                        break
+                    elif player in self.players[BLACK]:
+                        self.kill(BLACK_ENGINE_DIED)
+                        break
+
             log.debug("GameModel.run: emitting 'game_started' self=%s" % self)
             self.emit("game_started")
 
             # Let GameModel end() itself on games started with loadAndStart()
-            self.checkStatus()
+            if not self.lesson_game:
+                self.checkStatus()
 
             self.curColor = self.boards[-1].color
 
@@ -730,9 +765,23 @@ class GameModel(GObject.GObject):
                         spectator.putMove(self.boards[-1], self.moves[-1],
                                           self.boards[-2])
 
-                self.setOpening()
+                if self.puzzle_game and len(self.moves) % 2 == 1:
+                    status, reason = getStatus(self.boards[-1])
+                    self.failed_playing_best = self.check_failed_playing_best(status)
+                    if self.failed_playing_best:
+                        # print("failed_playing_best() == True -> yield from asyncio.sleep(1.5) ")
+                        # It may happen that analysis had no time to fill hints with best moves
+                        # so we give him another chance with some additional time to think on it
+                        self.spectators[HINT].setBoard(self.boards[-2])
+                        # TODO: wait for an event (analyzer PV reaching 18 ply)
+                        # instead of hard coded sleep time
+                        yield from asyncio.sleep(1.5)
+                        self.failed_playing_best = self.check_failed_playing_best(status)
 
                 self.checkStatus()
+
+                self.setOpening()
+
                 self.curColor = 1 - self.curColor
 
             self.checkStatus()
@@ -744,7 +793,6 @@ class GameModel(GObject.GObject):
             would return. That is, if the game is e.g. check mated this will
             call mode.end(), or if moves have been undone from an otherwise
             ended position, this will call __resume and emit game_unended. """
-
         log.debug("GameModel.checkStatus:")
 
         # call flag by engine
@@ -753,6 +801,9 @@ class GameModel(GObject.GObject):
 
         status, reason = getStatus(self.boards[-1])
 
+        if self.practice_game and (len(self.moves) % 2 == 1 or status in UNDOABLE_STATES):
+            self.check_goal(status, reason)
+
         if self.endstatus is not None:
             self.end(self.endstatus, reason)
             return
@@ -924,6 +975,14 @@ class GameModel(GObject.GObject):
         return False
 
     def add_variation(self, board, moves, comment="", score="", emit=True):
+        if board.board.next is None:
+            # If we are in the latest played board, and want to add a variation
+            # we have to add the latest move first
+            if board.board.lastMove is None or board.board.prev is None:
+                return
+            moves = [Move(board.board.lastMove)] + moves
+            board = board.board.prev.pieceBoard
+
         board0 = board
         board = board0.clone()
         board.board.prev = None
@@ -947,15 +1006,6 @@ class GameModel(GObject.GObject):
             variation.append(new)
             board = new
 
-        if board0.board.next is None:
-            # If we are in the latest played board, and want to add a variation
-            # we have to add a not played yet board first
-            # which can hold the variation as his child
-            from pychess.Utils.lutils.LBoard import LBoard
-            null_board = LBoard()
-            null_board.prev = board0.board
-            board0.board.next = null_board
-
         board0.board.next.children.append(
             [vboard.board for vboard in variation])
         if score:
@@ -996,3 +1046,46 @@ class GameModel(GObject.GObject):
         self.variations[variationIdx].append(new)
         self.needsSave = True
         self.emit("variation_extended", board.board, new.board)
+
+    def remove_variation(self, board, parent):
+        """ board must be an lboard object of the first Board object of a variation Board(!) list """
+        # Remove the variation (list of lboards) containing board from parent's children list
+        for child in parent.children:
+            if isinstance(child, list) and board in child:
+                parent.children.remove(child)
+                break
+
+        # Remove all variations from gamemodel's variations list which contains this board
+        for vari in self.variations[1:]:
+            if board.pieceBoard in vari:
+                self.variations.remove(vari)
+
+        # remove null_board if variation was added on last played move
+        if not parent.fen_was_applied:
+            parent.prev.next = None
+
+        self.needsSave = True
+
+    def undo_in_variation(self, board):
+        """ board must be the latest Board object of a variation board list """
+        assert board.board.next is None and len(board.board.children) == 0
+        self.emit("variation_undoing")
+
+        for vari in self.variations[1:]:
+            if board in vari:
+                break
+
+        board = board.board
+        parent = board.prev.next
+
+        # If this is a one move only variation we have to remove the whole variation
+        # if it's a longer one, just remove the latest move from it
+        first_vari_moves = [child[1] for child in parent.children if not isinstance(child, str)]
+        if board in first_vari_moves:
+            self.remove_variation(board, parent)
+        else:
+            board.prev.next = None
+            del vari[-1]
+
+        self.needsSave = True
+        self.emit("variation_undone")
