@@ -358,9 +358,7 @@ def parseSAN(board, san):
     else:
         piece = PAWN
         if notat[-1] in "18" and flag == NORMAL_MOVE and board.variant != SITTUYINCHESS:
-            raise ParsingError(
-                san, _("promotion move without promoted piece is incorrect"),
-                board.asFen())
+            flag = QUEEN_PROMOTION
 
     if "x" in notat:
         notat, tcord = notat.split("x")
@@ -633,9 +631,7 @@ def parseAN(board, an):
         flag = ENPASSANT
     elif board.arBoard[fcord] == PAWN:
         if an[3] in "18" and board.variant != SITTUYINCHESS:
-            raise ParsingError(
-                an, _("promotion move without promoted piece is incorrect"),
-                board.asFen())
+            flag = QUEEN_PROMOTION
 
     return newMove(fcord, tcord, flag)
 
@@ -653,6 +649,7 @@ san2WhiteFanDic = {
     ord("B"): FAN_PIECES[WHITE][BISHOP],
     ord("S"): FAN_PIECES[WHITE][BISHOP],
     ord("N"): FAN_PIECES[WHITE][KNIGHT],
+    ord("P"): FAN_PIECES[WHITE][PAWN],
     ord("+"): "†",
     ord("#"): "‡"
 }
@@ -666,6 +663,7 @@ san2BlackFanDic = {
     ord("B"): FAN_PIECES[BLACK][BISHOP],
     ord("S"): FAN_PIECES[BLACK][BISHOP],
     ord("N"): FAN_PIECES[BLACK][KNIGHT],
+    ord("P"): FAN_PIECES[BLACK][PAWN],
     ord("+"): "†",
     ord("#"): "‡"
 }
@@ -675,10 +673,7 @@ def toFAN(board, move):
     """ Returns a Figurine Algebraic Notation string of a move """
 
     san = toSAN(board, move)
-    if board.color == WHITE:
-        return san.translate(san2WhiteFanDic)
-    else:
-        return san.translate(san2BlackFanDic)
+    return san.translate(san2WhiteFanDic)
 
 
 ################################################################################
