@@ -12,7 +12,7 @@ import pychess
 from pychess.System.Log import log
 
 from pychess import ic
-from pychess.Utils.const import NAME, FISCHERRANDOMCHESS
+from pychess.Utils.const import NAME, FISCHERRANDOMCHESS, LOSERSCHESS, ATOMICCHESS, CRAZYHOUSECHESS
 from .managers.SeekManager import SeekManager
 from .managers.FingerManager import FingerManager
 from .managers.NewsManager import NewsManager
@@ -57,13 +57,14 @@ class Connection(GObject.GObject):
         'error': (GObject.SignalFlags.RUN_FIRST, None, (object, )),
     }
 
-    def __init__(self, host, ports, username, password):
+    def __init__(self, host, ports, timeseal, username, password):
         GObject.GObject.__init__(self)
         self.daemon = True
         self.host = host
         self.ports = ports
         self.username = username
         self.password = password
+        self.timeseal = timeseal
 
         self.connected = False
         self.connecting = False
@@ -170,8 +171,8 @@ PREVENTED = _("Due to abuse problems, guest connections have been prevented.\n"
 
 
 class FICSConnection(Connection):
-    def __init__(self, host, ports, username="guest", password=""):
-        Connection.__init__(self, host, ports, username, password)
+    def __init__(self, host, ports, timeseal, username="guest", password=""):
+        Connection.__init__(self, host, ports, timeseal, username, password)
 
     def _post_connect_hook(self, lines):
         pass
@@ -191,7 +192,7 @@ class FICSConnection(Connection):
                           extra={"task": (self.host, "raw")})
                 try:
                     connected_event = asyncio.Event()
-                    self.client = ICSTelnet()
+                    self.client = ICSTelnet(self.timeseal)
                     asyncio.async(self.client.start(self.host, port, connected_event))
                     yield from connected_event.wait()
                 except socket.error as err:
@@ -293,9 +294,15 @@ class FICSConnection(Connection):
                 self.client.lines.datagram_mode = True
 
                 ic.GAME_TYPES_BY_SHORT_FICS_NAME["B"] = ic.GAME_TYPES["bullet"]
+                ic.VARIANT_GAME_TYPES[ATOMICCHESS] = ic.GAME_TYPES["w27"]
+                ic.VARIANT_GAME_TYPES[CRAZYHOUSECHESS] = ic.GAME_TYPES["w23"]
+                ic.VARIANT_GAME_TYPES[LOSERSCHESS] = ic.GAME_TYPES["w17"]
                 ic.VARIANT_GAME_TYPES[FISCHERRANDOMCHESS] = ic.GAME_TYPES["w22"]
             else:
                 ic.GAME_TYPES_BY_SHORT_FICS_NAME["B"] = ic.GAME_TYPES["bughouse"]
+                ic.VARIANT_GAME_TYPES[ATOMICCHESS] = ic.GAME_TYPES["atomic"]
+                ic.VARIANT_GAME_TYPES[CRAZYHOUSECHESS] = ic.GAME_TYPES["crazyhouse"]
+                ic.VARIANT_GAME_TYPES[LOSERSCHESS] = ic.GAME_TYPES["losers"]
                 ic.VARIANT_GAME_TYPES[FISCHERRANDOMCHESS] = ic.GAME_TYPES["wild/fr"]
 
             self.client.run_command("iset defprompt 1")
@@ -346,18 +353,20 @@ class FICSConnection(Connection):
 
     def cancel(self):
         self.close()
-        self.client.cancel()
+        if hasattr(self, "client"):
+            self.client.cancel()
 
     def close(self):
         self.connected = False
-        self.client.close()
+        if hasattr(self, "client"):
+            self.client.close()
         if self.keep_alive_task is not None:
             self.keep_alive_task.cancel()
 
 
 class FICSMainConnection(FICSConnection):
-    def __init__(self, host, ports, username="guest", password=""):
-        FICSConnection.__init__(self, host, ports, username, password)
+    def __init__(self, host, ports, timeseal, username="guest", password=""):
+        FICSConnection.__init__(self, host, ports, timeseal, username, password)
         self.lvm = None
         self.notify_users = []
         self.ini_messages = []
@@ -372,11 +381,6 @@ class FICSMainConnection(FICSConnection):
         self.journal_owner = self.username
         self.set_user_vars = False
 
-        self.offline_lecture = False
-        self.lecture_skip_event = asyncio.Event()  # set when 'Go on' button pressed
-        self.lecture_pause_event = asyncio.Event()  # set when 'Pause' button pressed
-        self.lecture_exit_event = asyncio.Event()  # set when 'Exit' button pressed
-
     def close(self):
         if isinstance(self.client, PredictionsTelnet) and self.set_user_vars:
             self.client.run_command("set open 0")
@@ -477,8 +481,8 @@ class FICSMainConnection(FICSConnection):
 
 
 class FICSHelperConnection(FICSConnection):
-    def __init__(self, main_conn, host, ports, username="guest", password=""):
-        FICSConnection.__init__(self, host, ports, username, password)
+    def __init__(self, main_conn, host, ports, timeseal, username="guest", password=""):
+        FICSConnection.__init__(self, host, ports, timeseal, username, password)
         self.main_conn = main_conn
 
     def _start_managers(self, lines):
