@@ -75,6 +75,8 @@ class ICLogon(object):
             conf.set("hostEntry", self.host_get_value(widget), section=self.ics)
         self.widgets["hostEntry"].connect("changed", on_host_changed)
 
+        self.widgets["timesealCheck"].connect("toggled", self.on_timeseal_toggled)
+
         self.infobar = Gtk.InfoBar()
         self.infobar.set_message_type(Gtk.MessageType.WARNING)
         # self.widgets["messagePanelHBox"].pack_start(self.infobar,
@@ -94,7 +96,7 @@ class ICLogon(object):
         if value is not None:
             names = value.split("|")
         else:
-            names = conf.get("usernameEntry", "", section=self.ics).split("|")
+            names = conf.get("usernameEntry", section=self.ics).split("|")
         if len(names) == 0:
             names = ["", ""]
         elif len(names) < 2:
@@ -142,18 +144,22 @@ class ICLogon(object):
         self.widgets["passEntry"].set_sensitive(not widget.get_active())
         conf.set("asGuestCheck", widget.get_active(), section=self.ics)
 
+    def on_timeseal_toggled(self, widget):
+        conf.set("timesealCheck", widget.get_active(), section=self.ics)
+
     def on_ics_combo_changed(self, combo):
         tree_iter = combo.get_active_iter()
         if tree_iter is not None:
             model = combo.get_model()
             self.ics = model[tree_iter][0]
             # print("Selected: %s" % self.ics)
-            self.widgets["logOnAsGuest"].set_active(conf.get("asGuestCheck", True, section=self.ics))
+            self.widgets["logOnAsGuest"].set_active(conf.get("asGuestCheck", section=self.ics))
             self.on_logOnAsGuest_toggled(self.widgets["logOnAsGuest"])
-            self.user_name_set_value(self.widgets["nameEntry"], conf.get("usernameEntry", "", section=self.ics))
-            self.password_set_value(self.widgets["passEntry"], conf.get("passwordEntry", "", section=self.ics))
-            default_host = "freechess.org" if self.ics == "FICS" else "chessclub.com"
-            self.host_set_value(self.widgets["hostEntry"], conf.get("hostEntry", default_host, section=self.ics))
+            self.user_name_set_value(self.widgets["nameEntry"], conf.get("usernameEntry", section=self.ics))
+            self.password_set_value(self.widgets["passEntry"], conf.get("passwordEntry", section=self.ics))
+            self.host_set_value(self.widgets["hostEntry"], conf.get("hostEntry", section=self.ics))
+            self.widgets["timesealCheck"].set_active(conf.get("timesealCheck", section=self.ics))
+            self.on_timeseal_toggled(self.widgets["timesealCheck"])
 
     def _disconnect(self):
         for obj in self.cids:
@@ -198,7 +204,10 @@ class ICLogon(object):
 
         def pulse():
             self.widgets["progressbar"].pulse()
-            return not self.connection.isConnected()
+            if self.connection is None:
+                return False
+            else:
+                return not self.connection.isConnected()
 
         self.pulser = GObject.timeout_add(30, pulse)
 
@@ -292,21 +301,21 @@ class ICLogon(object):
                 ports.append(23)
         alternate_host = self.widgets["hostEntry"].get_text()
 
+        timeseal = self.widgets["timesealCheck"].get_active()
+
         self.showConnecting()
         self.host = host if host is not None else alternate_host if alternate_host else "freechess.org"
-        self.connection = FICSMainConnection(self.host, ports, username,
-                                             password)
+        self.connection = FICSMainConnection(self.host, ports, timeseal, username, password)
         for signal, callback in (("connected", self.onConnected),
                                  ("error", self.onConnectionError),
                                  ("connectingMsg", self.showMessage)):
-            self.cids[self.connection].append(self.connection.connect(
-                signal, callback))
+            self.cids[self.connection].append(self.connection.connect(signal, callback))
         self.main_connected_event = asyncio.Event()
         self.connection_task = asyncio.async(self.connection.start())
 
         # guest users are rather limited on ICC (helper connection is useless)
-        if self.host not in ("localhost", "chessclub.com"):
-            self.helperconn = FICSHelperConnection(self.connection, self.host, ports)
+        if self.host not in ("localhost", "127.0.0.1", "chessclub.com"):
+            self.helperconn = FICSHelperConnection(self.connection, self.host, ports, timeseal)
             self.helperconn.connect("error", self.onHelperConnectionError)
 
             @asyncio.coroutine
