@@ -1,7 +1,7 @@
 # -*- coding: UTF-8 -*-
 import ast
 
-from gi.repository import GLib, Gtk, GObject
+from gi.repository import GLib, Gdk, Gtk, GObject
 
 from pychess.Utils.const import chr2Sign, WHITE, BLACK
 from pychess.Utils.Piece import Piece
@@ -9,8 +9,10 @@ from pychess.Utils.SetupModel import SetupModel, SetupPlayer
 from pychess.System import uistuff
 from pychess.System.prefix import addDataPrefix
 from pychess.widgets.BoardControl import BoardControl
+from pychess.widgets.gameinfoDialog import on_pick_date
 from pychess.widgets.PieceWidget import PieceWidget
 from pychess.widgets import mainwindow
+from pychess.Variants import name2variant
 
 TAG_FILTER, MATERIAL_FILTER, PATTERN_FILTER, NONE, RULE, SEQUENCE, STREAK = range(7)
 
@@ -33,12 +35,32 @@ class FilterPanel(Gtk.TreeView):
         GObject.GObject.__init__(self)
         self.persp = persp
         self.filtered = False
+        self.widgets = uistuff.GladeWidgets("PyChess.glade")
+
+        # Build variant combo model
+        variant_store = Gtk.ListStore(str, int)
+
+        for name, variant in sorted(name2variant.items()):
+            variant_store.append((name, variant.variant))
+
+        self.widgets["variant"].set_model(variant_store)
+        renderer_text = Gtk.CellRendererText()
+        self.widgets["variant"].pack_start(renderer_text, True)
+        self.widgets["variant"].add_attribute(renderer_text, "text", 0)
+
+        # Connect date_from and date_to to corresponding calendars
+        self.widgets["date_from_button"].connect("clicked", on_pick_date, self.widgets["date_from"])
+        self.widgets["date_to_button"].connect("clicked", on_pick_date, self.widgets["date_to"])
 
         # Add piece widgets to dialog *_dock containers on material tab
-        self.widgets = uistuff.GladeWidgets("PyChess.glade")
         self.dialog = self.widgets["filter_dialog"]
         self.dialog.set_transient_for(mainwindow())
 
+        def hide(widget, event):
+            widget.hide()
+            return True
+        self.dialog.connect("delete-event", hide)
+
         for piece in "qrbnp":
             dock = "w%s_dock" % piece
             self.widgets[dock].add(PieceWidget(Piece(WHITE, chr2Sign[piece])))
@@ -61,6 +83,9 @@ class FilterPanel(Gtk.TreeView):
         self.widgets[dock].add(PieceWidget(Piece(BLACK, chr2Sign[piece])))
         self.widgets[dock].get_child().show()
 
+        self.widgets["copy_sub_fen"].connect("clicked", self.on_copy_sub_fen)
+        self.widgets["paste_sub_fen"].connect("clicked", self.on_paste_sub_fen)
+
         # We will store our filtering queries in a ListStore
         # column 0: query as text
         # column 1: query dict
@@ -216,7 +241,7 @@ class FilterPanel(Gtk.TreeView):
                 self.update_filters()
 
             if hasattr(self, "board_control"):
-                self.board_control.emit("action", "CLOSE", None)
+                self.board_control.emit("action", "CLOSE", None, None)
 
             self.dialog.hide()
             self.dialog.disconnect(handler_id)
@@ -270,7 +295,7 @@ class FilterPanel(Gtk.TreeView):
                 self.update_filters()
 
             if hasattr(self, "board_control"):
-                self.board_control.emit("action", "CLOSE", None)
+                self.board_control.emit("action", "CLOSE", None, None)
 
             self.dialog.hide()
             self.dialog.disconnect(handler_id)
@@ -368,13 +393,22 @@ class FilterPanel(Gtk.TreeView):
     def ini_widgets_from_query(self, query):
         """ Set filter dialog widget values based on query dict key-value pairs """
 
-        for rule in ("white", "black", "event", "site", "eco_from", "eco_to", "annotator"):
+        rule = "variant"
+        if rule in query:
+            index = 0
+            model = self.widgets["variant"].get_model()
+            for index, row in enumerate(model):
+                if query[rule] == row[1]:
+                    break
+            self.widgets["variant"].set_active(index)
+
+        for rule in ("white", "black", "event", "site", "date_from", "date_to", "eco_from", "eco_to", "annotator"):
             if rule in query:
                 self.widgets[rule].set_text(query[rule])
             else:
                 self.widgets[rule].set_text("")
 
-        for rule in ("elo_from", "elo_to", "year_from", "year_to"):
+        for rule in ("elo_from", "elo_to"):
             if rule in query:
                 self.widgets[rule].set_value(query[rule])
             else:
@@ -504,11 +538,17 @@ class FilterPanel(Gtk.TreeView):
         material_query = {}
         pattern_query = {}
 
-        for rule in ("white", "black", "event", "site", "eco_from", "eco_to", "annotator"):
+        tree_iter = self.widgets["variant"].get_active_iter()
+        if tree_iter is not None:
+            model = self.widgets["variant"].get_model()
+            variant_code = model[tree_iter][1]
+            tag_query["variant"] = variant_code
+
+        for rule in ("white", "black", "event", "site", "date_from", "date_to", "eco_from", "eco_to", "annotator"):
             if self.widgets[rule].get_text():
                 tag_query[rule] = self.widgets[rule].get_text()
 
-        for rule in ("elo_from", "elo_to", "year_from", "year_to"):
+        for rule in ("elo_from", "elo_to"):
             if self.widgets[rule].get_value_as_int():
                 tag_query[rule] = self.widgets[rule].get_value_as_int()
 
@@ -585,6 +625,18 @@ class FilterPanel(Gtk.TreeView):
     def fen_changed(self):
         self.widgets["sub_fen"].set_text(self.get_fen())
 
+    def on_copy_sub_fen(self, widget):
+        clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
+        text = self.widgets["sub_fen"].get_text()
+        if len(text) > 0:
+            clipboard.set_text(text, -1)
+
+    def on_paste_sub_fen(self, widget):
+        clipboard = Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)
+        text = clipboard.wait_for_text()
+        if text.count("/") == 7:
+            self.board_control.emit("action", "SETUP", None, text)
+
     def game_changed(self, model, ply):
         GLib.idle_add(self.fen_changed)
 
