@@ -5,7 +5,7 @@ import asyncio
 from gi.repository import Gtk, GObject
 
 from pychess.Utils.const import DRAW, LOCAL, WHITE, BLACK, WAITING_TO_START, reprResult, \
-    UNKNOWN_STATE, UNDOABLE_STATES, FIRST_PAGE, PREV_PAGE, NEXT_PAGE
+    UNDOABLE_STATES, FIRST_PAGE, PREV_PAGE, NEXT_PAGE
 from pychess.Players.Human import Human
 from pychess.Utils.GameModel import GameModel
 from pychess.perspectives import perspective_manager
@@ -14,7 +14,7 @@ from pychess.Database.model import game, event, site, pl1, pl2
 
 
 cols = (game.c.id, pl1.c.name, game.c.white_elo, pl2.c.name, game.c.black_elo,
-        game.c.result, game.c.date_year, event.c.name, site.c.name, game.c.round,
+        game.c.result, game.c.date, event.c.name, site.c.name, game.c.round,
         game.c.ply_count, game.c.eco, game.c.time_control, game.c.variant, game.c.fen)
 
 
@@ -43,7 +43,7 @@ class GameList(Gtk.TreeView):
 
         titles = (_("Id"), _("White"), _("W Elo"), _("Black"), _("B Elo"),
                   _("Result"), _("Date"), _("Event"), _("Site"), _("Round"),
-                  _("Length"), "ECO", "TC", _("Variant"), "FEN")
+                  _("Length"), "ECO", _("Time control"), _("Variant"), "FEN")
 
         for i, title in enumerate(titles):
             r = Gtk.CellRendererText()
@@ -64,12 +64,15 @@ class GameList(Gtk.TreeView):
         toolbar = Gtk.Toolbar()
 
         firstButton = Gtk.ToolButton(Gtk.STOCK_MEDIA_PREVIOUS)
+        firstButton.set_tooltip_text(_("First games"))
         toolbar.insert(firstButton, -1)
 
         prevButton = Gtk.ToolButton(Gtk.STOCK_MEDIA_REWIND)
+        prevButton.set_tooltip_text(_("Previous games"))
         toolbar.insert(prevButton, -1)
 
         nextButton = Gtk.ToolButton(Gtk.STOCK_MEDIA_FORWARD)
+        nextButton.set_tooltip_text(_("Next games"))
         toolbar.insert(nextButton, -1)
 
         firstButton.connect("clicked", self.on_first_clicked)
@@ -115,7 +118,6 @@ class GameList(Gtk.TreeView):
         else:
             self.liststore.clear()
 
-        get_date = self.persp.chessfile.get_date
         add = self.liststore.append
 
         self.records = []
@@ -125,16 +127,20 @@ class GameList(Gtk.TreeView):
             offs = rec["Offset"]
             wname = rec["White"]
             bname = rec["Black"]
-            welo = "" if rec["WhiteElo"] == 0 else str(rec["WhiteElo"])
-            belo = "" if rec["BlackElo"] == 0 else str(rec["BlackElo"])
+            welo = rec["WhiteElo"]
+            belo = rec["BlackElo"]
             result = rec["Result"]
             result = "½-½" if result == DRAW else reprResult[result] if result else "*"
-            event = rec["Event"]
-            site = rec["Site"]
-            round_ = rec["Round"]
-            date = str(get_date(rec))
-            ply = rec["PlyCount"]
-            length = str(int(ply) // 2) if ply else ""
+            event = "" if rec["Event"] is None else rec["Event"].replace("?", "")
+            site = "" if rec["Site"] is None else rec["Site"].replace("?", "")
+            round_ = "" if rec["Round"] is None else rec["Round"].replace("?", "")
+            date = "" if rec["Date"] is None else rec["Date"].replace(".??", "").replace("????.", "")
+
+            try:
+                ply = rec["PlyCount"]
+                length = str(int(ply) // 2) if ply else ""
+            except ValueError:
+                length = ""
             eco = rec["ECO"]
             tc = rec["TimeControl"]
             variant = rec["Variant"]
@@ -162,16 +168,18 @@ class GameList(Gtk.TreeView):
 
         self.gamemodel = GameModel()
 
-        variant = rec[13]
+        variant = rec["Variant"]
         if variant:
             self.gamemodel.tags["Variant"] = variant
 
-        wp, bp = rec["White"], rec["Black"]
+        # Lichess exports study .pgn without White and Black tags
+        wp = "" if rec["White"] is None else rec["White"]
+        bp = "" if rec["Black"] is None else rec["Black"]
         p0 = (LOCAL, Human, (WHITE, wp), wp)
         p1 = (LOCAL, Human, (BLACK, bp), bp)
         self.persp.chessfile.loadToModel(rec, -1, self.gamemodel)
 
-        self.gamemodel.endstatus = self.gamemodel.status if self.gamemodel.status in UNDOABLE_STATES else UNKNOWN_STATE
+        self.gamemodel.endstatus = self.gamemodel.status if self.gamemodel.status in UNDOABLE_STATES else None
         self.gamemodel.status = WAITING_TO_START
 
         perspective_manager.activate_perspective("games")
