@@ -208,7 +208,7 @@ class FICS(GObject.GObject, Perspective):
         self.connection.connect("disconnected", lambda connection: self.close())
         self.connection.connect("error", self.on_connection_error)
         if self.connection.isRegistred():
-            numtimes = conf.get("numberOfTimesLoggedInAsRegisteredUser", 0) + 1
+            numtimes = conf.get("numberOfTimesLoggedInAsRegisteredUser") + 1
             conf.set("numberOfTimesLoggedInAsRegisteredUser", numtimes)
         self.connection.em.connect("onCommandNotFound", lambda em, cmd: log.error(
             "Fics answered '%s': Command not found" % cmd))
@@ -227,7 +227,6 @@ class FICS(GObject.GObject, Perspective):
         self.connection.glm.connect("our-seeks-removed", self.our_seeks_removed)
         self.connection.cm.connect("arrivalNotification", self.onArrivalNotification)
         self.connection.cm.connect("departedNotification", self.onDepartedNotification)
-        self.connection.com.onConsoleMessage("", self.connection.ini_messages)
 
         def get_top_games():
             if perspective_manager.current_perspective == self:
@@ -265,6 +264,9 @@ class FICS(GObject.GObject, Perspective):
         if self.first_run:
             self.first_run = False
 
+        # After all panel is set up we can push initial messages out
+        self.connection.com.onConsoleMessage("", self.connection.ini_messages)
+
     def show(self):
         perspective_manager.activate_perspective("fics")
 
