@@ -15,7 +15,6 @@ from pychess.Utils.lutils.lmovegen import newMove
 from pychess.Utils.lutils.lmove import ParsingError
 from pychess.System.prefix import addDataPrefix
 from pychess.System.Log import log
-from pychess.widgets.BoardControl import play_or_add_move
 from math import ceil
 
 __title__ = _("Hints")
@@ -88,17 +87,18 @@ class Advisor(object):
 
 
 class OpeningAdvisor(Advisor):
-    def __init__(self, store, tv, boardview):
+    def __init__(self, store, tv, boardcontrol):
         Advisor.__init__(self, store, _("Opening Book"), OPENING)
         self.tooltip = _(
             "The opening book will try to inspire you during the opening phase of the game by showing you common moves made by chess masters")
         #        self.opening_names = []
         self.tv = tv
-        self.boardview = boardview
+        self.boardcontrol = boardcontrol
+        self.boardview = boardcontrol.view
 
     def shownChanged(self, boardview, shown):
         m = boardview.model
-        if m is None or m.variant.variant != NORMALCHESS or m.isPlayingICSGame():
+        if m is None or m.isPlayingICSGame():
             return
 
         b = m.getBoardAtPly(shown, boardview.shown_variation_idx)
@@ -142,12 +142,12 @@ class OpeningAdvisor(Advisor):
     def row_activated(self, iter, model, from_gui=True):
         if self.store.get_path(iter) != Gtk.TreePath(self.path):
             board, move, moves = self.store[iter][0]
-            play_or_add_move(self.boardview, board, move)
+            self.boardcontrol.play_or_add_move(board, move)
 
 
 class EngineAdvisor(Advisor):
     # An EngineAdvisor always has self.linesExpected rows reserved for analysis.
-    def __init__(self, store, engine, mode, tv, boardview):
+    def __init__(self, store, engine, mode, tv, boardcontrol):
         if mode == HINT:
             Advisor.__init__(self, store, _("Analysis by %s") % engine, HINT)
             self.tooltip = _(
@@ -161,14 +161,24 @@ class EngineAdvisor(Advisor):
         self.tv = tv
         self.active = False
         self.linesExpected = 1
-        self.boardview = boardview
+        self.boardview = boardcontrol.view
 
         self.cid1 = self.engine.connect("analyze", self.on_analyze)
         self.cid2 = self.engine.connect("readyForOptions", self.on_ready_for_options)
+        self.cid3 = self.engine.connect_after("readyForMoves", self.on_ready_for_moves)
+
+        self.figuresInNotation = conf.get("figuresInNotation")
+
+        def on_figures_in_notation(none):
+            self.figuresInNotation = conf.get("figuresInNotation")
+
+        self.cid4 = conf.notify_add("figuresInNotation", on_figures_in_notation)
 
     def _del(self):
         self.engine.disconnect(self.cid1)
         self.engine.disconnect(self.cid2)
+        self.engine.disconnect(self.cid3)
+        conf.notify_remove(self.cid4)
 
     def _create_new_expected_lines(self):
         parent = self.empty_parent()
@@ -181,11 +191,11 @@ class EngineAdvisor(Advisor):
         m = boardview.model
         if m is None:
             return
-        if m.isPlayingICSGame():
+        if m.isPlayingICSGame() and not m.lesson_game:
             return
 
         self.engine.setBoard(boardview.model.getBoardAtPly(
-            shown, boardview.shown_variation_idx), search=self.active)
+            shown, boardview.shown_variation_idx), search=self.active or m.lesson_game)
 
         if self.active:
             self._create_new_expected_lines()
@@ -209,6 +219,7 @@ class EngineAdvisor(Advisor):
         self.store.set_value(parent, 6, True)
         self.active = True
 
+    def on_ready_for_moves(self, engine):
         self.shownChanged(self.boardview, self.boardview.shown)
 
     def on_analyze(self, engine, analysis):
@@ -221,8 +232,6 @@ class EngineAdvisor(Advisor):
         if not self.active:
             return
 
-        is_FAN = conf.get("figuresInNotation", False)
-
         for i, line in enumerate(analysis):
             if line is None:
                 self.store[self.path + (i, )] = self.textOnlyRow("")
@@ -256,7 +265,7 @@ class EngineAdvisor(Advisor):
                     mvcount = ""
                 counted_pv.append("%s%s" %
                                   (mvcount, toFAN(board, pvmove)
-                                   if is_FAN else toSAN(board, pvmove, True)))
+                                   if self.figuresInNotation else toSAN(board, pvmove, True)))
                 board = board.move(pvmove)
 
             goodness = (min(max(score, -250), 250) + 250) / 500.0
@@ -265,7 +274,7 @@ class EngineAdvisor(Advisor):
 
             self.store[self.path + (i, )] = [
                 (board0, move, pv),
-                (prettyPrintScore(score, depth), 1, goodness), 0, False,
+                (prettyPrintScore(score, depth, format_mate=True), 1, goodness), 0, False,
                 " ".join(counted_pv), False, False
             ]
 
@@ -328,18 +337,18 @@ class EngineAdvisor(Advisor):
 
 
 class EndgameAdvisor(Advisor):
-    def __init__(self, store, tv, boardview):
+    def __init__(self, store, tv, boardcontrol):
         Advisor.__init__(self, store, _("Endgame Table"), ENDGAME)
         self.egtb = EndgameTable()
         # If mate in # was activated by double click let egtb do the rest
         self.auto_activate = False
         self.tv = tv
-        self.boardview = boardview
+        self.boardcontrol = boardcontrol
+        self.boardview = boardcontrol.view
         self.tooltip = _(
             "The endgame table will show exact analysis when there are few pieces on the board.")
         # TODO: Show a message if tablebases for the position exist but are neither installed nor allowed.
 
-        self.cid = self.egtb.connect("scored", self.on_scored)
         self.queue = asyncio.Queue()
         self.egtb_task = asyncio.async(self.start())
 
@@ -350,35 +359,33 @@ class EndgameAdvisor(Advisor):
     def start(self):
         while True:
             v = yield from self.queue.get()
-            if v == self.StopNow:
+            if isinstance(v, Exception) and v == self.StopNow:
                 break
             elif v == self.board.board:
-                self.egtb.scoreAllMoves(v)
+                ret = yield from self.egtb.scoreAllMoves(v)
+                self.on_scored(v, ret)
             self.queue.task_done()
 
     def shownChanged(self, boardview, shown):
         m = boardview.model
         if m is None or m.variant.variant != NORMALCHESS or m.isPlayingICSGame():
-            return
+            if not (m.practice_game or m.lesson_game):
+                return
 
         self.parent = self.empty_parent()
         self.board = m.getBoardAtPly(shown, boardview.shown_variation_idx)
         self.queue.put_nowait(self.board.board)
 
     def _del(self):
-        self.egtb.disconnect(self.cid)
         try:
             self.queue.put_nowait(self.StopNow)
         except asyncio.QueueFull:
             log.warning("EndgameAdvisor.gamewidget_closed: Queue.Full")
         self.egtb_task.cancel()
 
-    def on_scored(self, w, ret):
+    def on_scored(self, board, endings):
         m = self.boardview.model
-        if m.isPlayingICSGame():
-            return
 
-        board, endings = ret
         if board != self.board.board:
             return
 
@@ -392,8 +399,17 @@ class EndgameAdvisor(Advisor):
             else:
                 result = (_("Win"), 1, 1.0)
                 details = _("Mate in %d") % depth
+
+            if m.practice_game or m.lesson_game:
+                m.hint = "%s %s %s" % (toSAN(self.board, move, True), result[0], details)
+                return
+
+            if m.isPlayingICSGame():
+                return
+
             self.store.append(self.parent, [(self.board, move, None), result,
                                             0, False, details, False, False])
+
         self.tv.expand_row(Gtk.TreePath(self.path), False)
 
         if self.auto_activate:
@@ -415,14 +431,17 @@ class EndgameAdvisor(Advisor):
                     # double click on mate in #
                     self.auto_activate = True
 
-            play_or_add_move(self.boardview, board, move)
+            self.boardcontrol.play_or_add_move(board, move)
 
 
 class Sidepanel(object):
     def load(self, gmwidg):
         self.gmwidg = gmwidg
+        self.boardcontrol = gmwidg.board
         self.boardview = gmwidg.board.view
 
+        self.figuresInNotation = conf.get("figuresInNotation")
+
         self.sw = Gtk.ScrolledWindow()
         self.tv = Gtk.TreeView()
         self.tv.set_property("headers_visible", False)
@@ -445,7 +464,7 @@ class Sidepanel(object):
             if not move:
                 cell.set_property("text", "")
             else:
-                if conf.get("figuresInNotation", False):
+                if self.figuresInNotation:
                     cell.set_property("text", toFAN(board, move))
                 else:
                     cell.set_property("text", toSAN(board, move, True))
@@ -496,7 +515,7 @@ class Sidepanel(object):
             if store[iter][5]:
                 cell.set_property("stock-id", "gtk-media-play")
             else:
-                cell.set_property("stock-id", "gtk-stop")
+                cell.set_property("stock-id", "gtk-media-pause")
 
         c4.set_cell_data_func(self.toggleRenderer, cb_visible)
 
@@ -526,11 +545,11 @@ class Sidepanel(object):
         self.advisors = []
         self.conf_conids = []
 
-        if conf.get("opening_check", False):
-            advisor = OpeningAdvisor(self.store, self.tv, self.boardview)
+        if conf.get("opening_check"):
+            advisor = OpeningAdvisor(self.store, self.tv, self.boardcontrol)
             self.advisors.append(advisor)
-        if conf.get("endgame_check", False):
-            advisor = EndgameAdvisor(self.store, self.tv, self.boardview)
+        if conf.get("endgame_check"):
+            advisor = EndgameAdvisor(self.store, self.tv, self.boardcontrol)
             self.advisors.append(advisor)
 
         self.model_cids = [
@@ -540,8 +559,8 @@ class Sidepanel(object):
         ]
 
         def on_opening_check(none):
-            if conf.get("opening_check", False) and self.boardview is not None:
-                advisor = OpeningAdvisor(self.store, self.tv, self.boardview)
+            if conf.get("opening_check") and self.boardview is not None:
+                advisor = OpeningAdvisor(self.store, self.tv, self.boardcontrol)
                 self.advisors.append(advisor)
                 advisor.shownChanged(self.boardview, self.boardview.shown)
             else:
@@ -554,8 +573,7 @@ class Sidepanel(object):
         self.conf_conids.append(conf.notify_add("opening_check", on_opening_check))
 
         def on_opening_file_entry_changed(none):
-            default_path = os.path.join(addDataPrefix("pychess_book.bin"))
-            path = conf.get("opening_file_entry", default_path)
+            path = conf.get("opening_file_entry")
             if os.path.isfile(path):
                 for advisor in self.advisors:
                     if advisor.mode == OPENING and self.boardview is not None:
@@ -565,8 +583,8 @@ class Sidepanel(object):
         self.conf_conids.append(conf.notify_add("opening_file_entry", on_opening_file_entry_changed))
 
         def on_endgame_check(none):
-            if conf.get("endgame_check", False):
-                advisor = EndgameAdvisor(self.store, self.tv, self.boardview)
+            if conf.get("endgame_check"):
+                advisor = EndgameAdvisor(self.store, self.tv, self.boardcontrol)
                 self.advisors.append(advisor)
                 advisor.shownChanged(self.boardview, self.boardview.shown)
             else:
@@ -579,6 +597,11 @@ class Sidepanel(object):
 
         self.conf_conids.append(conf.notify_add("endgame_check", on_endgame_check))
 
+        def on_figures_in_notation(none):
+            self.figuresInNotation = conf.get("figuresInNotation")
+
+        self.conf_conids.append(conf.notify_add("figuresInNotation", on_figures_in_notation))
+
         return self.sw
 
     def on_game_terminated(self, model):
@@ -597,10 +620,10 @@ class Sidepanel(object):
     def on_analyzer_added(self, gamemodel, analyzer, analyzer_type):
         if analyzer_type == HINT:
             self.advisors.append(EngineAdvisor(self.store, analyzer, HINT,
-                                               self.tv, self.boardview))
+                                               self.tv, self.boardcontrol))
         if analyzer_type == SPY:
             self.advisors.append(EngineAdvisor(self.store, analyzer, SPY,
-                                               self.tv, self.boardview))
+                                               self.tv, self.boardcontrol))
 
     def on_analyzer_removed(self, gamemodel, analyzer, analyzer_type):
         for advisor in self.advisors:
