@@ -19,40 +19,6 @@ from .PromotionDialog import PromotionDialog
 from .BoardView import BoardView, rect, join
 
 
-def play_or_add_move(view, board, move):
-    if board.board.next is None:
-        # at the end of variation or main line
-        if not view.shownIsMainLine():
-            # add move to existing variation
-            view.model.add_move2variation(
-                board, move, view.shown_variation_idx)
-            view.shown += 1
-        else:
-            # create new variation
-            new_vari = view.model.add_variation(board, (move, ))
-            view.setShownBoard(new_vari[-1])
-    else:
-        # inside variation or main line
-        if board.board.next.lastMove == move.move:
-            # replay mainline move
-            view.shown += 1
-        elif board.board.next.children:
-            # try to find this move in variations
-            for i, vari in enumerate(board.board.next.children):
-                for node in vari:
-                    if type(node) != str and node.lastMove == move.move and node.plyCount == board.ply + 1:
-                        # replay variation move
-                        view.setShownBoard(node.pieceBoard)
-                        return
-            # create new variation
-            new_vari = view.model.add_variation(board, (move, ))
-            view.setShownBoard(new_vari[-1])
-        else:
-            # create new variation
-            new_vari = view.model.add_variation(board, (move, ))
-            view.setShownBoard(new_vari[-1])
-
-
 class BoardControl(Gtk.EventBox):
     """ Creates a BoardView for GameModel to control move selection,
         action menu selection and emits signals to let Human player
@@ -62,14 +28,16 @@ class BoardControl(Gtk.EventBox):
     """
 
     __gsignals__ = {
+        'shapes_changed': (GObject.SignalFlags.RUN_FIRST, None, ()),
         'piece_moved': (GObject.SignalFlags.RUN_FIRST, None, (object, int)),
-        'action': (GObject.SignalFlags.RUN_FIRST, None, (str, object))
+        'action': (GObject.SignalFlags.RUN_FIRST, None, (str, object, object))
     }
 
     def __init__(self, gamemodel, action_menu_items, setup_position=False, game_preview=False):
         GObject.GObject.__init__(self)
         self.setup_position = setup_position
         self.game_preview = game_preview
+
         self.view = BoardView(gamemodel, setup_position=setup_position)
         self.add(self.view)
         self.variant = gamemodel.variant
@@ -92,6 +60,7 @@ class BoardControl(Gtk.EventBox):
         self.gamemodel_cids = []
         self.gamemodel_cids.append(gamemodel.connect("moves_undoing", self.moves_undone))
         self.gamemodel_cids.append(gamemodel.connect("game_ended", self.game_ended))
+        self.gamemodel_cids.append(gamemodel.connect("game_started", self.game_started))
 
         self.cids = []
         self.cids.append(self.connect("button_press_event", self.button_press))
@@ -161,13 +130,96 @@ class BoardControl(Gtk.EventBox):
         promotion = self.promotionDialog.runAndHide(color, variant)
         return promotion
 
+    def play_sound(self, move, board):
+        if move.is_capture(board):
+            sound = "aPlayerCaptures"
+        else:
+            sound = "aPlayerMoves"
+
+        if board.board.isChecked():
+            sound = "aPlayerChecks"
+
+        preferencesDialog.SoundTab.playAction(sound)
+
+    def play_or_add_move(self, board, move):
+        if board.board.next is None:
+            # at the end of variation or main line
+            if not self.view.shownIsMainLine():
+                # add move to existing variation
+                self.view.model.add_move2variation(board, move, self.view.shown_variation_idx)
+                self.view.showNext()
+            else:
+                # create new variation
+                new_vari = self.view.model.add_variation(board, [move])
+                self.view.setShownBoard(new_vari[-1])
+        else:
+            # inside variation or main line
+            if board.board.next.lastMove == move.move:
+                # replay mainline move
+                if self.view.model.lesson_game:
+                    next_board = self.view.model.getBoardAtPly(self.view.shown + 1, self.view.shown_variation_idx)
+                    self.play_sound(move, board)
+                    incr = 1 if len(self.view.model.variations[self.view.shown_variation_idx]) - 1 == board.ply - self.view.model.lowply + 1 else 2
+                    if incr == 2:
+                        next_next_board = self.view.model.getBoardAtPly(self.view.shown + 2, self.view.shown_variation_idx)
+                        # If there is any opponent move variation let the user choose opp next move
+                        if any(child for child in next_next_board.board.children if isinstance(child, list)):
+                            self.view.infobar.opp_turn()
+                            self.view.showNext()
+                        # If there is some comment to read let the user read it before opp move
+                        elif any(child for child in next_board.board.children if isinstance(child, str)):
+                            self.view.infobar.opp_turn()
+                            self.view.showNext()
+
+                        # If there is nothing to wait for we make opp next move
+                        else:
+                            self.view.showNext()
+                            self.view.infobar.your_turn()
+                            self.view.showNext()
+                    else:
+                        if self.view.shownIsMainLine():
+                            preferencesDialog.SoundTab.playAction("puzzleSuccess")
+                            self.view.infobar.get_next_puzzle()
+                            self.view.model.emit("learn_success")
+                            self.view.showNext()
+                        else:
+                            self.view.infobar.back_to_mainline()
+                            self.view.showNext()
+                else:
+                    self.view.showNext()
+
+            elif board.board.next.children:
+                if self.view.model.lesson_game:
+                    self.play_sound(move, board)
+                    self.view.infobar.retry()
+
+                # try to find this move in variations
+                for i, vari in enumerate(board.board.next.children):
+                    for node in vari:
+                        if type(node) != str and node.lastMove == move.move and node.plyCount == board.ply + 1:
+                            # replay variation move
+                            self.view.setShownBoard(node.pieceBoard)
+                            return
+
+                # create new variation
+                new_vari = self.view.model.add_variation(board, [move])
+                self.view.setShownBoard(new_vari[-1])
+
+            else:
+                if self.view.model.lesson_game:
+                    self.play_sound(move, board)
+                    self.view.infobar.retry()
+
+                # create new variation
+                new_vari = self.view.model.add_variation(board, [move])
+                self.view.setShownBoard(new_vari[-1])
+
     def emit_move_signal(self, cord0, cord1, promotion=None):
         # Game end can change cord0 to None while dragging a piece
         if cord0 is None:
             return
-        color = self.view.model.boards[-1].color
-        board = self.view.model.getBoardAtPly(self.view.shown,
-                                              self.view.shown_variation_idx)
+        board = self.getBoard()
+        color = board.color
         # Ask player for which piece to promote into. If this move does not
         # include a promotion, QUEEN will be sent as a dummy value, but not used
         if promotion is None and board[cord0].sign == PAWN and \
@@ -185,7 +237,7 @@ class BoardControl(Gtk.EventBox):
             elif len(self.variant.PROMOTIONS) == 1:
                 promotion = lmove.PROMOTE_PIECE(self.variant.PROMOTIONS[0])
             else:
-                if conf.get("autoPromote", False):
+                if conf.get("autoPromote"):
                     promotion = lmove.PROMOTE_PIECE(QUEEN_PROMOTION)
                 else:
                     promotion = self.getPromotion()
@@ -210,36 +262,43 @@ class BoardControl(Gtk.EventBox):
                 if self.view.model.examined:
                     self.view.model.connection.bm.sendMove(toAN(board, move))
         else:
-            play_or_add_move(self.view, board, move)
+            self.play_or_add_move(board, move)
 
     def actionActivate(self, widget, key):
         """ Put actions from a menu or similar """
+        curplayer = self.view.model.curplayer
         if key == "call_flag":
-            self.emit("action", FLAG_CALL, None)
+            self.emit("action", FLAG_CALL, curplayer, None)
         elif key == "abort":
-            self.emit("action", ABORT_OFFER, None)
+            self.emit("action", ABORT_OFFER, curplayer, None)
         elif key == "adjourn":
-            self.emit("action", ADJOURN_OFFER, None)
+            self.emit("action", ADJOURN_OFFER, curplayer, None)
         elif key == "draw":
-            self.emit("action", DRAW_OFFER, None)
+            self.emit("action", DRAW_OFFER, curplayer, None)
         elif key == "resign":
-            self.emit("action", RESIGNATION, None)
+            self.emit("action", RESIGNATION, curplayer, None)
         elif key == "ask_to_move":
-            self.emit("action", HURRY_ACTION, None)
+            self.emit("action", HURRY_ACTION, curplayer, None)
         elif key == "undo1":
-            curplayer = self.view.model.curplayer
+            board = self.view.model.getBoardAtPly(self.view.shown, variation=self.view.shown_variation_idx)
+            if board.board.next is not None or board.board.children:
+                return
+            if not self.view.shownIsMainLine():
+                self.view.model.undo_in_variation(board)
+                return
+
             waitingplayer = self.view.model.waitingplayer
             if curplayer.__type__ == LOCAL and \
                     (waitingplayer.__type__ == ARTIFICIAL or
                      self.view.model.isPlayingICSGame()) and \
                     self.view.model.ply - self.view.model.lowply > 1:
-                self.emit("action", TAKEBACK_OFFER, self.view.model.ply - 2)
+                self.emit("action", TAKEBACK_OFFER, curplayer, 2)
             else:
-                self.emit("action", TAKEBACK_OFFER, self.view.model.ply - 1)
+                self.emit("action", TAKEBACK_OFFER, curplayer, 1)
         elif key == "pause1":
-            self.emit("action", PAUSE_OFFER, None)
+            self.emit("action", PAUSE_OFFER, curplayer, None)
         elif key == "resume1":
-            self.emit("action", RESUME_OFFER, None)
+            self.emit("action", RESUME_OFFER, curplayer, None)
 
     def shownChanged(self, view, shown):
         if self.view is None:
@@ -270,6 +329,15 @@ class BoardControl(Gtk.EventBox):
 
         self.view.startAnimation()
 
+    def game_started(self, gamemodel):
+        if self.view.model.lesson_game:
+            if "FEN" in gamemodel.tags:
+                if gamemodel.orientation != gamemodel.starting_color:
+                    self.view.showNext()
+            else:
+                self.view.infobar.get_next_puzzle()
+                self.view.model.emit("learn_success")
+
     def getBoard(self):
         return self.view.model.getBoardAtPly(self.view.shown,
                                              self.view.shown_variation_idx)
@@ -333,10 +401,23 @@ class BoardControl(Gtk.EventBox):
             self.view.setPremove(None, None, None, None)
             self.currentState = self.normalState
 
+    def color(self, event):
+        state = event.get_state()
+        if state & Gdk.ModifierType.SHIFT_MASK and state & Gdk.ModifierType.CONTROL_MASK:
+            return "Y"
+        elif state & Gdk.ModifierType.SHIFT_MASK:
+            return "R"
+        elif state & Gdk.ModifierType.CONTROL_MASK:
+            return "B"
+        else:
+            return "G"
+
     def button_press(self, widget, event):
         if event.button == 3:
             # first we will draw a circle
-            cord = self.currentState.point2Cord(event.x, event.y)
+            cord = self.currentState.point2Cord(event.x, event.y, self.color(event))
+            if cord is None or cord.x < 0 or cord.x > self.FILES or cord.y < 0 or cord.y > self.RANKS:
+                return
             self.pre_arrow_from = cord
             self.view.pre_circle = cord
             self.view.redrawCanvas()
@@ -366,13 +447,16 @@ class BoardControl(Gtk.EventBox):
     def button_release(self, widget, event):
         if event.button == 3:
             # remove or finalize circle/arrow as needed
-            cord = self.currentState.point2Cord(event.x, event.y)
+            cord = self.currentState.point2Cord(event.x, event.y, self.color(event))
+            if cord is None or cord.x < 0 or cord.x > self.FILES or cord.y < 0 or cord.y > self.RANKS:
+                return
             if self.view.pre_circle == cord:
                 if cord in self.view.circles:
                     self.view.circles.remove(cord)
                 else:
                     self.view.circles.add(cord)
                 self.view.pre_circle = None
+                self.emit("shapes_changed")
 
             if self.view.pre_arrow is not None:
                 if self.view.pre_arrow in self.view.arrows:
@@ -380,6 +464,7 @@ class BoardControl(Gtk.EventBox):
                 else:
                     self.view.arrows.add(self.view.pre_arrow)
                 self.view.pre_arrow = None
+                self.emit("shapes_changed")
 
             self.pre_arrow_from = None
             self.pre_arrow_to = None
@@ -392,6 +477,8 @@ class BoardControl(Gtk.EventBox):
 
     def motion_notify(self, widget, event):
         to = self.currentState.point2Cord(event.x, event.y)
+        if to is None or to.x < 0 or to.x > self.FILES or to.y < 0 or to.y > self.RANKS:
+            return
         if self.pre_arrow_from is not None:
             if to != self.pre_arrow_from:
                 # this will be an arrow
@@ -452,7 +539,7 @@ class BoardControl(Gtk.EventBox):
                     if self.view.model.examined:
                         self.view.model.connection.bm.sendMove(toAN(board, move))
                 else:
-                    play_or_add_move(self.view, board, move)
+                    self.play_or_add_move(board, move)
             self.keybuffer = ""
 
         elif keyname == "BackSpace":
@@ -538,7 +625,7 @@ class BoardState:
         x_loc /= float(side)
         return x_loc, self.RANKS - y_loc
 
-    def point2Cord(self, x_loc, y_loc):
+    def point2Cord(self, x_loc, y_loc, color=None):
         point = self.transPoint(x_loc, y_loc)
         p0_loc, p1_loc = point[0], point[1]
         if self.parent.variant.variant in DROP_VARIANTS:
@@ -549,7 +636,7 @@ class BoardState:
             if not 0 <= int(p0_loc) <= self.FILES - 1 or not 0 <= int(
                     p1_loc) <= self.RANKS - 1:
                 return None
-        return Cord(int(p0_loc) if p0_loc >= 0 else int(p0_loc) - 1, int(p1_loc))
+        return Cord(int(p0_loc) if p0_loc >= 0 else int(p0_loc) - 1, int(p1_loc), color)
 
     def isSelectable(self, cord):
         # Simple isSelectable method, disabling selecting cords out of bound etc
@@ -874,7 +961,7 @@ class LockedActiveState(LockedBoardState):
             board = self.getBoard()
             if board[self.view.selected].sign == PAWN and \
                     cord.cord in board.PROMOTION_ZONE[1 - board.color]:
-                if conf.get("autoPromote", False):
+                if conf.get("autoPromote"):
                     promotion = lmove.PROMOTE_PIECE(QUEEN_PROMOTION)
                 else:
                     promotion = self.parent.getPromotion()
@@ -893,7 +980,7 @@ class LockedActiveState(LockedBoardState):
             board = self.getBoard()
             if board[self.view.active].sign == PAWN and \
                     cord.cord in board.PROMOTION_ZONE[1 - board.color]:
-                if conf.get("autoPromote", False):
+                if conf.get("autoPromote"):
                     promotion = lmove.PROMOTE_PIECE(QUEEN_PROMOTION)
                 else:
                     promotion = self.parent.getPromotion()
