@@ -16,6 +16,7 @@ from pychess.Utils.const import WAITING_TO_START, WHITE, BLACK, WHITEWON, \
     UNDOABLE_REASONS, BLACK_ENGINE_DIED, HINT, SPY, RUNNING, ABORT_OFFER, ADJOURN_OFFER, \
     DRAW_OFFER, PAUSE_OFFER, RESUME_OFFER, HURRY_ACTION, FLAG_CALL
 from pychess.Utils.repr import reprResult_long, reprReason_long
+from pychess.Utils.LearnModel import LearnModel
 from pychess.System import conf
 from pychess.System.Log import log
 from pychess.widgets import preferencesDialog
@@ -150,7 +151,7 @@ class GameNanny(object):
                     gmwidg.cids[player] = player.connect("notify::status", status_changed, button)
                     status_changed(player, None, button)
 
-        elif gamemodel.hasLocalPlayer():
+        elif gamemodel.hasLocalPlayer() and not isinstance(gamemodel, LearnModel):
 
             def callback(infobar, response, message, gamemodel=gamemodel):
                 if response == 1:
@@ -159,9 +160,9 @@ class GameNanny(object):
                     createRematch(gamemodel)
                 elif response == 2:
                     if gamemodel.ply > 1:
-                        offer = Offer(TAKEBACK_OFFER, gamemodel.ply - 2)
+                        offer = Offer(TAKEBACK_OFFER, 2)
                     else:
-                        offer = Offer(TAKEBACK_OFFER, gamemodel.ply - 1)
+                        offer = Offer(TAKEBACK_OFFER, 1)
                     if gamemodel.players[0].__type__ == LOCAL:
                         gamemodel.players[0].emit("offer", offer)
                     else:
@@ -190,25 +191,34 @@ class GameNanny(object):
             self.engineDead(gamemodel.players[1], gmwidg)
 
         if (isinstance(gamemodel, ICGameModel) and not gamemodel.isObservationGame()) or \
-                gamemodel.isEngine2EngineGame():
+                gamemodel.isEngine2EngineGame() or \
+                (isinstance(gamemodel, LearnModel) and not gamemodel.failed_playing_best):
             asyncio.async(gamemodel.restart_analyzer(HINT))
             asyncio.async(gamemodel.restart_analyzer(SPY))
-            if not conf.get("hint_mode", False):
+            if not conf.get("hint_mode"):
                 gamemodel.pause_analyzer(HINT)
-            if not conf.get("spy_mode", False):
+            if not conf.get("spy_mode"):
                 gamemodel.pause_analyzer(SPY)
 
         return False
 
     def on_game_started(self, gamemodel, gmwidg):
+        # offline lectures can reuse same gamemodel/gamewidget
+        # to show several examples inside the same lecture
+        if gamemodel.offline_lecture:
+            gmwidg.clearMessages()
+
         # Rotate to human player
         boardview = gmwidg.board.view
         if gamemodel.players[1].__type__ == LOCAL:
             if gamemodel.players[0].__type__ != LOCAL:
                 boardview.rotation = math.pi
-            elif conf.get("autoRotate", True) and \
-                    gamemodel.curplayer == gamemodel.players[1]:
+
+        if isinstance(gamemodel, LearnModel):
+            if gamemodel.orientation == BLACK:
                 boardview.rotation = math.pi
+            else:
+                boardview.rotation = 0
 
         # Play set-up sound
         preferencesDialog.SoundTab.playAction("gameIsSetup")
@@ -223,9 +233,9 @@ class GameNanny(object):
         if not gamemodel.isEngine2EngineGame():
             asyncio.async(gamemodel.start_analyzer(HINT))
             asyncio.async(gamemodel.start_analyzer(SPY))
-            if not conf.get("hint_mode", False):
+            if not conf.get("hint_mode"):
                 gamemodel.pause_analyzer(HINT)
-            if not conf.get("spy_mode", False):
+            if not conf.get("spy_mode"):
                 gamemodel.pause_analyzer(SPY)
         return False
 
