@@ -1,3 +1,5 @@
+# -*- coding: UTF-8 -*-
+
 import asyncio
 import os.path
 import gettext
@@ -15,13 +17,14 @@ from pychess.Utils.IconLoader import load_icon, get_pixbuf
 from pychess.Utils.GameModel import GameModel
 from pychess.Utils.SetupModel import SetupModel, SetupPlayer
 from pychess.Utils.TimeModel import TimeModel
-from pychess.Utils.const import FISCHERRANDOMCHESS, LOSERSCHESS, NORMALCHESS, VARIANTS_BLINDFOLD, \
+from pychess.Utils.const import NORMALCHESS, VARIANTS_BLINDFOLD, FISCHERRANDOMCHESS, \
     VARIANTS_ODDS, VARIANTS_SHUFFLE, VARIANTS_OTHER, VARIANTS_OTHER_NONSTANDARD, VARIANTS_ASEAN, \
-    WHITE, BLACK, UNSUPPORTED, ARTIFICIAL, LOCAL, reprFile, W_OO, W_OOO, B_OO, B_OOO, \
+    WHITE, BLACK, UNSUPPORTED, ARTIFICIAL, LOCAL, reprCord, reprFile, W_OO, W_OOO, B_OO, B_OOO, \
     FAN_PIECES, reprSign, FEN_START, WAITING_TO_START
 
 from pychess.Utils.repr import localReprSign
 from pychess.Utils.lutils.ldata import FILE
+from pychess.Utils.lutils.LBoard import LBoard
 from pychess.System import uistuff
 from pychess.System.Log import log
 from pychess.System.protoopen import splitUri
@@ -142,66 +145,57 @@ class _GameInitializationMode(object):
         cls.widgets["blackPlayerButton"].set_image(Gtk.Image.new_from_pixbuf(cls.black))
         cls.widgets["blackPlayerButton"].connect("button-press-event", on_exchange_players)
 
-        uistuff.createCombo(cls.widgets["whitePlayerCombobox"],
-                            name="whitePlayerCombobox")
-        uistuff.createCombo(cls.widgets["blackPlayerCombobox"],
-                            name="blackPlayerCombobox")
+        uistuff.createCombo(cls.widgets["whitePlayerCombobox"], name="whitePlayerCombobox")
+        uistuff.createCombo(cls.widgets["blackPlayerCombobox"], name="blackPlayerCombobox")
 
         cls.widgets["playersIcon"].set_from_pixbuf(big_people)
         cls.widgets["timeIcon"].set_from_pixbuf(big_time)
 
-        def on_playerCombobox_changed(widget, skill_hbox):
-            skill_hbox.props.visible = widget.get_active() > 0
+        def on_playerCombobox_changed(widget, skill_hbox, skill_level):
+            position = widget.get_active()
+            skill_hbox.props.visible = position > 0
+            if position > 0:
+                tree_iter = widget.get_active_iter()
+                if tree_iter is not None:
+                    engine_name = widget.get_model()[tree_iter][1]
+                    engine = discoverer.getEngineByName(engine_name)
+                    if engine:
+                        pref_level = engine.get("level")
+                        if pref_level:
+                            skill_level.set_value(pref_level)
 
-        cls.widgets["whitePlayerCombobox"].connect(
-            "changed", on_playerCombobox_changed, cls.widgets["skillHbox1"])
-        cls.widgets["blackPlayerCombobox"].connect(
-            "changed", on_playerCombobox_changed, cls.widgets["skillHbox2"])
+        cls.widgets["whitePlayerCombobox"].connect("changed", on_playerCombobox_changed, cls.widgets["skillHbox1"], cls.widgets["skillSlider1"])
+        cls.widgets["blackPlayerCombobox"].connect("changed", on_playerCombobox_changed, cls.widgets["skillHbox2"], cls.widgets["skillSlider2"])
         cls.widgets["whitePlayerCombobox"].set_active(0)
         cls.widgets["blackPlayerCombobox"].set_active(1)
 
         def on_skill_changed(scale, image):
             image.set_from_pixbuf(skillToIcon[int(scale.get_value())])
 
-        cls.widgets["skillSlider1"].connect("value-changed", on_skill_changed,
-                                            cls.widgets["skillIcon1"])
-        cls.widgets["skillSlider2"].connect("value-changed", on_skill_changed,
-                                            cls.widgets["skillIcon2"])
+        cls.widgets["skillSlider1"].connect("value-changed", on_skill_changed, cls.widgets["skillIcon1"])
+        cls.widgets["skillSlider2"].connect("value-changed", on_skill_changed, cls.widgets["skillIcon2"])
         cls.widgets["skillSlider1"].set_value(3)
         cls.widgets["skillSlider2"].set_value(3)
 
-        cls.__initTimeRadio(
-            _("Blitz"), "ngblitz", cls.widgets["blitzRadio"],
-            cls.widgets["configImageBlitz"], 5, 0, 0)
-        cls.__initTimeRadio(
-            _("Rapid"), "ngrapid", cls.widgets["rapidRadio"],
-            cls.widgets["configImageRapid"], 15, 5, 0)
-        cls.__initTimeRadio(
-            _("Normal"), "ngnormal", cls.widgets["normalRadio"],
-            cls.widgets["configImageNormal"], 40, 15, 0)
-        cls.__initTimeRadio(
-            _("Classical"), "ngclassical", cls.widgets["classicalRadio"],
-            cls.widgets["configImageClassical"], 3, 0, 40)
+        cls.__initTimeRadio("ngblitz", cls.widgets["blitzRadio"], cls.widgets["configImageBlitz"], 5, 0, 0)
+        cls.__initTimeRadio("ngrapid", cls.widgets["rapidRadio"], cls.widgets["configImageRapid"], 15, 5, 0)
+        cls.__initTimeRadio("ngnormal", cls.widgets["normalRadio"], cls.widgets["configImageNormal"], 45, 15, 0)
+        cls.__initTimeRadio("ngclassical", cls.widgets["classicalRadio"], cls.widgets["configImageClassical"], 3, 0, 40)
 
-        cls.__initVariantRadio("ngvariant1", cls.widgets["playVariant1Radio"],
-                               cls.widgets["configImageVariant1"],
-                               FISCHERRANDOMCHESS)
-        cls.__initVariantRadio("ngvariant2", cls.widgets["playVariant2Radio"],
-                               cls.widgets["configImageVariant2"], LOSERSCHESS)
+        cls.__initVariantRadio("ngvariant1", cls.widgets["playVariant1Radio"], cls.widgets["configImageVariant1"])
+        cls.__initVariantRadio("ngvariant2", cls.widgets["playVariant2Radio"], cls.widgets["configImageVariant2"])
 
         def updateCombos(*args):
             if cls.widgets["playNormalRadio"].get_active():
                 variant = NORMALCHESS
             elif cls.widgets["playVariant1Radio"].get_active():
-                variant = conf.get("ngvariant1", FISCHERRANDOMCHESS)
+                variant = conf.get("ngvariant1")
             else:
-                variant = conf.get("ngvariant2", LOSERSCHESS)
-            variant1 = conf.get("ngvariant1", FISCHERRANDOMCHESS)
-            cls.widgets["playVariant1Radio"].set_tooltip_text(variants[
-                variant1].__desc__)
-            variant2 = conf.get("ngvariant2", LOSERSCHESS)
-            cls.widgets["playVariant2Radio"].set_tooltip_text(variants[
-                variant2].__desc__)
+                variant = conf.get("ngvariant2")
+            variant1 = conf.get("ngvariant1")
+            cls.widgets["playVariant1Radio"].set_tooltip_text(variants[variant1].__desc__)
+            variant2 = conf.get("ngvariant2")
+            cls.widgets["playVariant2Radio"].set_tooltip_text(variants[variant2].__desc__)
             data = [(item[0], item[1]) for item in playerItems[variant]]
             uistuff.updateCombo(cls.widgets["blackPlayerCombobox"], data)
             uistuff.updateCombo(cls.widgets["whitePlayerCombobox"], data)
@@ -212,24 +206,20 @@ class _GameInitializationMode(object):
         conf.notify_add("ngvariant1", updateCombos)
         conf.notify_add("ngvariant2", updateCombos)
         cls.widgets["playNormalRadio"].connect("toggled", updateCombos)
-        cls.widgets["playNormalRadio"].set_tooltip_text(variants[
-            NORMALCHESS].__desc__)
+        cls.widgets["playNormalRadio"].set_tooltip_text(variants[NORMALCHESS].__desc__)
         cls.widgets["playVariant1Radio"].connect("toggled", updateCombos)
-        variant1 = conf.get("ngvariant1", FISCHERRANDOMCHESS)
-        cls.widgets["playVariant1Radio"].set_tooltip_text(variants[
-            variant1].__desc__)
+        variant1 = conf.get("ngvariant1")
+        cls.widgets["playVariant1Radio"].set_tooltip_text(variants[variant1].__desc__)
         cls.widgets["playVariant2Radio"].connect("toggled", updateCombos)
-        variant2 = conf.get("ngvariant2", LOSERSCHESS)
-        cls.widgets["playVariant2Radio"].set_tooltip_text(variants[
-            variant2].__desc__)
+        variant2 = conf.get("ngvariant2")
+        cls.widgets["playVariant2Radio"].set_tooltip_text(variants[variant2].__desc__)
 
         # The "variant" has to come before players, because the engine positions
         # in the user comboboxes can be different in different variants
         for key in ("whitePlayerCombobox", "blackPlayerCombobox",
                     "skillSlider1", "skillSlider2", "notimeRadio",
                     "blitzRadio", "rapidRadio", "normalRadio", "classicalRadio",
-                    "playNormalRadio", "playVariant1Radio",
-                    "playVariant2Radio"):
+                    "playNormalRadio", "playVariant1Radio", "playVariant2Radio"):
             uistuff.keep(cls.widgets[key], key)
 
         # We don't want the dialog to deallocate when closed. Rather we hide
@@ -237,20 +227,19 @@ class _GameInitializationMode(object):
         cls.widgets["newgamedialog"].connect("delete_event", lambda *a: True)
 
     @classmethod
-    def __initTimeRadio(cls, name, id, radiobutton, configImage, defmin,
-                        defgain, defmoves):
+    def __initTimeRadio(cls, id, radiobutton, configImage, defmin, defgain, defmoves):
         minSpin = Gtk.SpinButton()
         minSpin.set_adjustment(Gtk.Adjustment(1, 0, 240, 1))
         setattr(cls, "%s_min" % id, minSpin)
-        uistuff.keep(minSpin, "%s min" % id, first_value=defmin)
+        uistuff.keep(minSpin, "%s min" % id)
         movesSpin = Gtk.SpinButton()
         movesSpin.set_adjustment(Gtk.Adjustment(0, 0, 60, 20))
         setattr(cls, "%s_moves" % id, movesSpin)
-        uistuff.keep(movesSpin, "%s moves" % id, first_value=defmoves)
+        uistuff.keep(movesSpin, "%s moves" % id)
         gainSpin = Gtk.SpinButton()
         gainSpin.set_adjustment(Gtk.Adjustment(0, -60, 60, 1))
         setattr(cls, "%s_gain" % id, gainSpin)
-        uistuff.keep(gainSpin, "%s gain" % id, first_value=defgain)
+        uistuff.keep(gainSpin, "%s gain" % id)
 
         table = Gtk.Table(2, 2)
         table.props.row_spacing = 3
@@ -272,42 +261,72 @@ class _GameInitializationMode(object):
         ImageMenu.switchWithImage(configImage, alignment)
 
         def updateString(spin):
+            # Elements of the clock
             minutes = minSpin.get_value_as_int()
             gain = gainSpin.get_value_as_int()
             moves = movesSpin.get_value_as_int()
-            if gain > 0:
-                radiobutton.set_label(
-                    _("%(name)s %(minutes)d min + %(gain)d sec/move") % {
-                        'name': name,
-                        'minutes': minutes,
-                        'gain': gain
-                    })
-            elif gain < 0:
-                radiobutton.set_label(
-                    _("%(name)s %(minutes)d min %(gain)d sec/move") % {
-                        'name': name,
-                        'minutes': minutes,
-                        'gain': gain
-                    })
-            elif moves > 0:
-                radiobutton.set_label(_("%(name)s %(minutes)d min / %(moves)d moves") % {
-                    'name': name,
+
+            # Duration of the game
+            def calculate_duration(ref_moves):
+                if moves > 0:
+                    return int(2 * minutes * ref_moves / moves)
+                else:
+                    return max(0, int(2 * minutes + ref_moves * gain / 30))
+
+            duration_20 = calculate_duration(20)
+            duration_40 = calculate_duration(40)
+            duration_60 = calculate_duration(60)
+
+            # Determination of the caption
+            def get_game_name():
+                """ https://www.fide.com/fide/handbook.html?id=171&view=article """
+                if defmoves > 0:
+                    return _("Classical")
+                if duration_60 <= 20:  # 10 minutes per player
+                    return _("Blitz")
+                if duration_60 < 120:  # 60 minutes per player
+                    return _("Rapid")
+                return _("Normal")
+
+            if moves > 0:
+                radiobutton.set_label(_("%(name)s %(minutes)d min / %(moves)d moves %(duration)s") % {
+                    'name': get_game_name(),
                     'minutes': minutes,
                     'moves': moves,
+                    'duration': ("(%d')" % duration_40) if duration_40 > 0 else ""
                 })
+            elif gain != 0:
+                radiobutton.set_label(
+                    _("%(name)s %(minutes)d min %(sign)s %(gain)d sec/move %(duration)s") % {
+                        'name': get_game_name(),
+                        'minutes': minutes,
+                        'sign': "+" if gain > 0 else "–",
+                        'gain': abs(gain),
+                        'duration': ("(%d')" % duration_40) if duration_40 > 0 else ""
+                    })
             else:
-                radiobutton.set_label(_("%(name)s %(minutes)d min") % {
-                    'name': name,
-                    'minutes': minutes
+                radiobutton.set_label(_("%(name)s %(minutes)d min %(duration)s") % {
+                    'name': get_game_name(),
+                    'minutes': minutes,
+                    'duration': ("(%d')" % duration_40) if duration_40 > 0 else ""
                 })
 
+            # Determination of the tooltip
+            if duration_20 > 0 and duration_60 > 0 and duration_20 != duration_60:
+                radiobutton.set_tooltip_text(_("Estimated duration : %(min)d - %(max)d minutes") % ({
+                    "min": min(duration_20, duration_60),
+                    "max": max(duration_20, duration_60)}
+                ))
+            else:
+                radiobutton.set_tooltip_text("")
+
         minSpin.connect("value-changed", updateString)
         movesSpin.connect("value-changed", updateString)
         gainSpin.connect("value-changed", updateString)
         updateString(None)
 
     @classmethod
-    def __initVariantRadio(cls, confid, radiobutton, configImage, default):
+    def __initVariantRadio(cls, confid, radiobutton, configImage):
         model = Gtk.TreeStore(str)
         treeview = Gtk.TreeView(model)
         treeview.set_headers_visible(False)
@@ -352,7 +371,7 @@ class _GameInitializationMode(object):
             return path.get_depth() > 1
 
         selection.set_select_function(selfunc, None)
-        variant = conf.get(confid, default)
+        variant = conf.get(confid)
         if variant in variantToPath:
             selection.select_path(variantToPath[variant])
 
@@ -376,7 +395,7 @@ class _GameInitializationMode(object):
                 # print("put clipboard:", clipboard.wait_for_text())
                 return
             elif response == CLEAR:
-                cls.board_control.emit("action", "SETUP", True)
+                cls.board_control.emit("action", "SETUP", None, True)
                 cls.ini_widgets(True)
                 # print("clear")
                 return
@@ -389,7 +408,7 @@ class _GameInitializationMode(object):
                 try:
                     lboard = cls.setupmodel.variant(setup=text).board
                     cls.ini_widgets(lboard.asFen())
-                    cls.board_control.emit("action", "SETUP", text)
+                    cls.board_control.emit("action", "SETUP", None, text)
                 except SyntaxError as e:
                     d = Gtk.MessageDialog(mainwindow(), type=Gtk.MessageType.WARNING,
                                           buttons=Gtk.ButtonsType.OK,
@@ -402,7 +421,7 @@ class _GameInitializationMode(object):
             elif response == INITIAL:
                 lboard = cls.setupmodel.variant(setup=FEN_START).board
                 cls.ini_widgets(lboard.asFen())
-                cls.board_control.emit("action", "SETUP", FEN_START)
+                cls.board_control.emit("action", "SETUP", None, FEN_START)
                 return
             elif response != Gtk.ResponseType.OK:
                 cls.widgets["newgamedialog"].hide()
@@ -410,15 +429,15 @@ class _GameInitializationMode(object):
                 return
 
             if hasattr(cls, "board_control"):
-                cls.board_control.emit("action", "CLOSE", None)
+                cls.board_control.emit("action", "CLOSE", None, None)
 
             # Find variant
             if cls.widgets["playNormalRadio"].get_active():
                 variant_index = NORMALCHESS
             elif cls.widgets["playVariant1Radio"].get_active():
-                variant_index = conf.get("ngvariant1", FISCHERRANDOMCHESS)
+                variant_index = conf.get("ngvariant1")
             else:
-                variant_index = conf.get("ngvariant2", LOSERSCHESS)
+                variant_index = conf.get("ngvariant2")
             variant = variants[variant_index]
 
             # Find time
@@ -475,9 +494,9 @@ class _GameInitializationMode(object):
                                         incr, moves], name))
                 else:
                     if not playertups or playertups[0][0] != LOCAL:
-                        name = conf.get("firstName", conf.username)
+                        name = conf.get("firstName")
                     else:
-                        name = conf.get("secondName", _("Guest"))
+                        name = conf.get("secondName")
                     playertups.append((LOCAL, Human, (color, name), name))
 
             # Set forcePonderOff initPlayerEngine param True in engine-engine games
@@ -568,10 +587,11 @@ class SetupPositionExtension(_GameInitializationMode):
                                            cls.moveno_spin_changed)
         cls.widgets["fifty_spin"].connect("value-changed",
                                           cls.fifty_spin_changed)
-        cls.widgets["woo"].connect("toggled", cls.castl_toggled, "K")
-        cls.widgets["wooo"].connect("toggled", cls.castl_toggled, "Q")
-        cls.widgets["boo"].connect("toggled", cls.castl_toggled, "k")
-        cls.widgets["booo"].connect("toggled", cls.castl_toggled, "q")
+
+        cls.widgets["woo"].connect("toggled", cls.castl_toggled, W_OO)
+        cls.widgets["wooo"].connect("toggled", cls.castl_toggled, W_OOO)
+        cls.widgets["boo"].connect("toggled", cls.castl_toggled, B_OO)
+        cls.widgets["booo"].connect("toggled", cls.castl_toggled, B_OOO)
 
         ep_store = Gtk.ListStore(str)
         ep_store.append(["-"])
@@ -624,10 +644,30 @@ class SetupPositionExtension(_GameInitializationMode):
 
     @classmethod
     def castl_toggled(cls, button, castl):
-        if button.get_active():
-            cls.castl.add(castl)
+        lboard = cls.setupmodel.boards[-1].board
+        if lboard.variant == FISCHERRANDOMCHESS:
+            if castl == W_OO:
+                cast_letter = reprCord[lboard.ini_rooks[0][1]][0].upper()
+            elif castl == W_OOO:
+                cast_letter = reprCord[lboard.ini_rooks[0][0]][0].upper()
+            elif castl == B_OO:
+                cast_letter = reprCord[lboard.ini_rooks[1][1]][0]
+            elif castl == B_OOO:
+                cast_letter = reprCord[lboard.ini_rooks[1][0]][0]
         else:
-            cls.castl.discard(castl)
+            if castl == W_OO:
+                cast_letter = "K"
+            elif castl == W_OOO:
+                cast_letter = "Q"
+            elif castl == B_OO:
+                cast_letter = "k"
+            elif castl == B_OOO:
+                cast_letter = "q"
+
+        if button.get_active():
+            cls.castl.add(cast_letter)
+        else:
+            cls.castl.discard(cast_letter)
         cls.fen_changed()
 
     @classmethod
@@ -651,8 +691,9 @@ class SetupPositionExtension(_GameInitializationMode):
         return "%s %s %s %s %s %s" % parts
 
     @classmethod
-    def ini_widgets(cls, setup):
-        lboard = cls.setupmodel.variant(setup=setup).board
+    def ini_widgets(cls, setup, lboard=None):
+        if lboard is None:
+            lboard = cls.setupmodel.variant(setup=setup).board
         cls.widgets["side_button"].set_active(False if lboard.color == WHITE
                                               else True)
         cls.widgets["fifty_spin"].set_value(lboard.fifty)
@@ -665,7 +706,7 @@ class SetupPositionExtension(_GameInitializationMode):
         cls.widgets["booo"].set_active(lboard.castling & B_OOO)
 
     @classmethod
-    def run(cls, fenstr):
+    def run(cls, fenstr, variant):
         cls._ensureReady()
         if cls.widgets["newgamedialog"].props.visible:
             cls.widgets["newgamedialog"].present()
@@ -690,9 +731,11 @@ class SetupPositionExtension(_GameInitializationMode):
         cls.widgets["setupBoardDock"].add(cls.board_control)
         cls.board_control.show_all()
         if fenstr is not None:
-            cls.setupmodel.boards = [cls.setupmodel.variant(setup=fenstr)]
+            lboard = LBoard(variant)
+            lboard.applyFen(fenstr)
+            cls.setupmodel.boards = [cls.setupmodel.variant(setup=fenstr, lboard=lboard)]
             cls.setupmodel.variations = [cls.setupmodel.boards]
-            cls.ini_widgets(fenstr)
+            cls.ini_widgets(fenstr, lboard)
         else:
             fenstr = cls.get_fen()
             cls.ini_widgets(True)
@@ -833,13 +876,13 @@ class EnterNotationExtension(_GameInitializationMode):
                 player0combo = cls.widgets["whitePlayerCombobox"]
                 player0 = player0combo.get_active()
                 if player0 == 0 and '[White "' not in text:
-                    name = '[White "%s"]' % conf.get("firstName", conf.username)
+                    name = '[White "%s"]' % conf.get("firstName")
                     text = "%s\n%s" % (name, text)
 
                 player1combo = cls.widgets["blackPlayerCombobox"]
                 player1 = player1combo.get_active()
                 if player1 == 0 and '[Black "' not in text:
-                    name = '[Black "%s"]' % conf.get("secondName", _("Guest"))
+                    name = '[Black "%s"]' % conf.get("secondName")
                     text = "%s\n%s" % (name, text)
 
                 loadType = pgn
